@@ -76,7 +76,8 @@ val group: CicNotationPt.term list -> CicNotationPt.term
 val ungroup: CicNotationPt.term list -> CicNotationPt.term list
 
 val find_appl_pattern_uris:
-  CicNotationPt.cic_appl_pattern -> UriManager.uri list
+  CicNotationPt.cic_appl_pattern ->
+   [`Uri of UriManager.uri | `NRef of NReference.reference] list
 
 val find_branch:
   CicNotationPt.term -> CicNotationPt.term
