@@ -26,7 +26,7 @@
 val procedural_of_acic_object:
    ids_to_inner_sorts:(Cic.id, Cic2acic.sort_kind) Hashtbl.t ->
    ids_to_inner_types:(Cic.id, Cic2acic.anntypes) Hashtbl.t -> ?info:string ->
-   ?depth:int -> ?flavour:Cic.object_flavour -> string -> Cic.annobj ->
+   GrafiteAst.inline_param list -> Cic.annobj ->
       (Cic.annterm, Cic.annterm,
        Cic.annterm GrafiteAst.reduction, Cic.annterm CicNotationPt.obj, string)
       GrafiteAst.statement list
@@ -34,9 +34,11 @@ val procedural_of_acic_object:
 val procedural_of_acic_term:
    ids_to_inner_sorts:(Cic.id, Cic2acic.sort_kind) Hashtbl.t ->
    ids_to_inner_types:(Cic.id, Cic2acic.anntypes) Hashtbl.t -> 
-   ?depth:int -> string -> Cic.context -> Cic.annterm ->
+   GrafiteAst.inline_param list -> Cic.context -> Cic.annterm ->
       (Cic.annterm, Cic.annterm,
        Cic.annterm GrafiteAst.reduction, Cic.annterm CicNotationPt.obj, string)
       GrafiteAst.statement list
 
-val debug: bool ref
+val tex_formatter: Format.formatter option ref
+
+val is_debug: int -> GrafiteAst.inline_param list -> bool
