@@ -27,8 +27,10 @@ type dependences = (CicInspect.S.t * bool) list
 
 type conclusion = (int * int * UriManager.uri * int) option
 
+val make: 'a list -> dependences
+
 val classify: Cic.context -> Cic.term -> dependences * conclusion
 
 val adjust: Cic.context -> Cic.annterm list -> ?goal:Cic.term -> dependences -> dependences
 
-val to_string: dependences * conclusion -> string
+val to_string: CicInspect.S.t -> dependences * conclusion -> string
