@@ -25,8 +25,10 @@
 
 type status
 
+val suffix: string
+
 val init: unit -> unit
 
-val make: string -> string -> status
+val make: string -> status
 
 val produce: status -> unit
