@@ -23,6 +23,6 @@
  * http://cs.unibo.it/helm/.
  *)
 
-val commit: out_channel -> Types.items -> unit
+val commit: Types.output_kind -> out_channel -> Types.items -> unit
 
 val string_of_inline_kind: Types.inline_kind -> string
