@@ -35,7 +35,7 @@
 (*                                                                           *)
 (*****************************************************************************)
 
-(* $Id: cic.ml 8654 2008-05-30 19:28:41Z tassi $ *)
+(* $Id: cic.ml 9486 2009-01-13 10:54:34Z tassi $ *)
 
 (* STUFF TO MANAGE IDENTIFIERS *)
 type id = string  (* the abstract type of the (annotated) node identifiers *)
@@ -67,8 +67,7 @@ type object_flavour =
   ]
 
 type object_class =
-  [ `Coercion of int
-  | `Elim of sort   (** elimination principle; if sort is Type, the universe is
+  [ `Elim of sort   (** elimination principle; if sort is Type, the universe is
                       * not relevant *)
   | `Record of (string * bool * int) list (** 
                         inductive type that encodes a record; the arguments are
