@@ -31,5 +31,8 @@ val get_rels_from_premise: int -> Cic.term -> S.t
 
 val get_mutinds_of_uri: UriManager.uri -> Cic.term -> S.t
 
-(* count_nodes n t returns n + the number of nodes in t *)
-val count_nodes: int -> Cic.term -> int
+(* count_nodes n t returns n + the number of nodes in t                *)
+(* implicits, metas and casts are counted if ~meta:true                *)
+(* if ~meta:false, complies with                                       *)
+(* F.Guidi: Procedural representation of CIC Proof Terms. Last version *)
+val count_nodes: meta:bool -> int -> Cic.term -> int
