@@ -23,4 +23,5 @@
  * http://cs.unibo.it/helm/.
  *)
 
-val unshare : Cic.term -> Cic.term
+val unshare : ?fresh_univs:bool -> Cic.term -> Cic.term
+val fresh_types: Cic.obj -> Cic.obj
