@@ -23,7 +23,7 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: cicSubstitution.ml 8490 2008-05-03 21:31:45Z sacerdot $ *)
+(* $Id: cicSubstitution.ml 9271 2008-11-28 18:28:58Z fguidi $ *)
 
 exception CannotSubstInMeta;;
 exception RelToHiddenHypothesis;;
@@ -40,7 +40,7 @@ let debug_print =
   fun _  -> ()
 ;;
 
-let lift_from k n =
+let lift_map k map =
  let rec liftaux k =
   let module C = Cic in
    function
@@ -48,7 +48,7 @@ let lift_from k n =
        if m < k then
         C.Rel m
        else
-        C.Rel (m + n)
+        C.Rel (map k m)
     | C.Var (uri,exp_named_subst) ->
        let exp_named_subst' = 
         List.map (function (uri,t) -> (uri,liftaux k t)) exp_named_subst
@@ -108,6 +108,9 @@ let lift_from k n =
  in
  liftaux k
 
+let lift_from k n =
+   lift_map k (fun _ m -> m + n)
+
 let lift n t =
   if n = 0 then
    t
