@@ -42,8 +42,10 @@ val parse_level2_meta: Ulexing.lexbuf -> CicNotationPt.term
 
 type rule_id
 
-val check_l1_pattern: (* level1_pattern *)
- CicNotationPt.term -> int -> Gramext.g_assoc -> checked_l1_pattern
+val compare_rule_id : rule_id -> rule_id -> int
+
+val check_l1_pattern: (* level1_pattern, pponly, precedence, assoc *)
+ CicNotationPt.term -> bool ->  int -> Gramext.g_assoc -> checked_l1_pattern
 
 val extend:
   checked_l1_pattern ->
@@ -55,15 +57,15 @@ val delete: rule_id -> unit
 (** {2 Grammar entries}
  * needed by grafite parser *)
 
-val level2_ast_grammar: Grammar.g
+val level2_ast_grammar: unit -> Grammar.g
 
-val term : CicNotationPt.term Grammar.Entry.e
+val term : unit -> CicNotationPt.term Grammar.Entry.e
 
-val let_defs :
+val let_defs : unit ->
   (CicNotationPt.term CicNotationPt.capture_variable list * CicNotationPt.term CicNotationPt.capture_variable * CicNotationPt.term * int) list
     Grammar.Entry.e
 
-val protected_binder_vars :
+val protected_binder_vars : unit ->
   (CicNotationPt.term list * CicNotationPt.term option) Grammar.Entry.e
 
 val parse_term: Ulexing.lexbuf -> CicNotationPt.term
@@ -73,3 +75,5 @@ val parse_term: Ulexing.lexbuf -> CicNotationPt.term
   (** print "level2_pattern" entry on stdout, flushing afterwards *)
 val print_l2_pattern: unit -> unit
 
+val push: unit -> unit
+val pop: unit -> unit
