@@ -38,3 +38,9 @@ val content2pres:
   Cic.annterm Content.cobj ->
     CicNotationPres.boxml_markup
 
+val ncontent2pres:
+  ?skip_initial_lambdas:int -> ?skip_thm_and_qed:bool ->
+  ids_to_nrefs:(NTermCicContent.id, NReference.reference) Hashtbl.t ->
+  CicNotationPt.term Content.cobj ->
+    CicNotationPres.boxml_markup
+
