@@ -32,7 +32,7 @@
 (*                                                                         *)
 (***************************************************************************)
 
-(* $Id: sequent2pres.ml 7772 2007-10-28 21:56:23Z sacerdot $ *)
+(* $Id: sequent2pres.ml 9819 2009-06-03 08:56:58Z sacerdot $ *)
 
 let p_mtr a b = Mpresentation.Mtr(a,b)
 let p_mtd a b = Mpresentation.Mtd(a,b)
@@ -47,7 +47,7 @@ let b_ink a = Box.Ink a
 module K = Content
 module P = Mpresentation
 
-let sequent2pres term2pres (_,_,context,ty) =
+let sequent2pres0 term2pres (_,_,context,ty) =
    let context2pres context = 
      let rec aux accum =
      function 
@@ -96,12 +96,25 @@ let sequent2pres term2pres (_,_,context,ty) =
        pres_goal]))])
 
 let sequent2pres ~ids_to_inner_sorts =
-  sequent2pres
+  sequent2pres0
     (fun annterm ->
       let ast, ids_to_uris =
        TermAcicContent.ast_of_acic ~output_type:`Term ids_to_inner_sorts annterm
       in
       CicNotationPres.box_of_mpres
-        (CicNotationPres.render ids_to_uris
+        (CicNotationPres.render
+          ~lookup_uri:(CicNotationPres.lookup_uri ids_to_uris)
           (TermContentPres.pp_ast ast)))
 
+let nsequent2pres ~ids_to_nrefs ~subst =
+ let lookup_uri id =
+  try
+   let nref = Hashtbl.find ids_to_nrefs id in
+    Some (NReference.string_of_reference nref)
+  with Not_found -> None
+ in
+  sequent2pres0
+    (fun ast ->
+      CicNotationPres.box_of_mpres
+       (CicNotationPres.render ~lookup_uri
+         (TermContentPres.pp_ast ast)))
