@@ -48,5 +48,5 @@ val instantiate_level2:
   CicNotationEnv.t -> CicNotationPt.term ->
     CicNotationPt.term
 
-(* hack. seee cicNotation for explanation *)
-val reset: unit -> unit
+val push: unit -> unit
+val pop: unit -> unit
