@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: test_lexer.ml 7950 2007-11-27 16:56:54Z sacerdot $ *)
+(* $Id: test_lexer.ml 9021 2008-09-26 16:38:29Z tassi $ *)
 
 let _ =
   let level = ref "2@" in
@@ -37,9 +37,9 @@ let _ =
   Arg.parse arg_spec open_file usage;
   let lexer =
     match !level with
-	"1" -> CicNotationLexer.level1_pattern_lexer
-      | "2@" -> CicNotationLexer.level2_ast_lexer
-      | "2$" -> CicNotationLexer.level2_meta_lexer
+	"1" -> CicNotationLexer.level1_pattern_lexer ()
+      | "2@" -> CicNotationLexer.level2_ast_lexer ()
+      | "2$" -> CicNotationLexer.level2_meta_lexer ()
       | l ->
 	  prerr_endline (Printf.sprintf "Unsupported level %s" l);
 	  exit 2
