@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: graphvizPp.ml 6743 2006-09-04 15:51:36Z tassi $ *)
+(* $Id: graphvizPp.ml 10520 2009-10-28 12:19:04Z tassi $ *)
 
 type attribute = string * string  (* <key, value> pair *)
 
@@ -69,7 +69,7 @@ module Dot =
 
     let node = node ~quote:true
     let edge name1 name2 ?(attrs = []) fmt =
-      fprintf fmt "@[<hov2>\"%s\" ->@ \"%s\"@ [" name1 name2;
+      fprintf fmt "@[<hov2>%s ->@ %s@ [" name1 name2;
       attributes attrs fmt;
       fprintf fmt "];@]@,"
     let raw s fmt = pp_print_string fmt s
