@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: grafiteAstPp.ml 9486 2009-01-13 10:54:34Z tassi $ *)
+(* $Id: grafiteAstPp.ml 10564 2009-11-16 17:09:31Z ricciott $ *)
 
 open GrafiteAst
 
@@ -89,6 +89,60 @@ let pp_just ~term_pp =
   | `Auto params -> pp_auto_params ~term_pp params
 ;;
 
+let rec pp_ntactic ~map_unicode_to_tex =
+ let term_pp = CicNotationPp.pp_term in
+ let lazy_term_pp = fun _ -> assert false in
+ let pp_tactic_pattern =
+  pp_tactic_pattern ~map_unicode_to_tex ~lazy_term_pp ~term_pp in
+ function
+  | NApply (_,t) -> "napply " ^ CicNotationPp.pp_term t
+  | NAuto (_,(l,flgs)) ->
+      "nauto" ^ 
+        (if l <> [] then (" by " ^
+         (String.concat "," (List.map CicNotationPp.pp_term l))) else "") ^
+        String.concat " " (List.map (fun a,b -> a ^ "=" ^ b) flgs)
+  | NCases (_,what,where) -> "ncases " ^ CicNotationPp.pp_term what ^
+      assert false ^ " " ^ assert false
+  | NConstructor (_,None,l) -> "@ " ^
+      String.concat " " (List.map CicNotationPp.pp_term l)
+  | NConstructor (_,Some x,l) -> "@" ^ string_of_int x ^ " " ^
+      String.concat " " (List.map CicNotationPp.pp_term l)
+  | NCase1 (_,n) -> "*" ^ n ^ ":"
+  | NChange (_,what,wwhat) -> "nchange " ^ assert false ^ 
+      " with " ^ CicNotationPp.pp_term wwhat
+  | NCut (_,t) -> "ncut " ^ CicNotationPp.pp_term t
+(*| NDiscriminate (_,t) -> "ndiscriminate " ^ CicNotationPp.pp_term t
+  | NSubst (_,t) -> "nsubst " ^ CicNotationPp.pp_term t *)
+  | NDestruct _ -> "ndestruct"
+  | NElim (_,what,where) -> "nelim " ^ CicNotationPp.pp_term what ^
+      assert false ^ " " ^ assert false
+  | NId _ -> "nid"
+  | NIntro (_,n) -> "#" ^ n
+  | NLApply (_,t) -> "lapply " ^ CicNotationPp.pp_term t
+  | NRewrite (_,dir,n,where) -> "nrewrite " ^
+     (match dir with `LeftToRight -> ">" | `RightToLeft -> "<") ^
+     " " ^ CicNotationPp.pp_term n ^ " " ^ pp_tactic_pattern where
+  | NReduce _ | NGeneralize _ | NLetIn _ | NAssert _ -> "TO BE IMPLEMENTED"
+  | NDot _ -> "##."
+  | NSemicolon _ -> "##;"
+  | NBranch _ -> "##["
+  | NShift _ -> "##|"
+  | NPos (_, l) -> "##" ^String.concat "," (List.map string_of_int l)^ ":"
+  | NPosbyname (_, s) -> "##" ^ s ^ ":"
+  | NWildcard _ -> "##*:"
+  | NMerge _ -> "##]"
+  | NFocus (_,l) -> 
+      Printf.sprintf "##focus %s" 
+        (String.concat " " (List.map string_of_int l))
+  | NUnfocus _ -> "##unfocus"
+  | NSkip _ -> "##skip"
+  | NTry (_,tac) -> "ntry " ^ pp_ntactic ~map_unicode_to_tex tac
+  | NAssumption _ -> "nassumption"
+  | NBlock (_,l) -> 
+     "(" ^ String.concat " " (List.map (pp_ntactic ~map_unicode_to_tex) l)^ ")"
+  | NRepeat (_,t) -> "nrepeat " ^ pp_ntactic ~map_unicode_to_tex t
+;;
+
 let rec pp_tactic ~map_unicode_to_tex ~term_pp ~lazy_term_pp =
  let pp_terms = pp_terms ~term_pp in
  let pp_tactics = pp_tactics ~map_unicode_to_tex ~term_pp ~lazy_term_pp in
@@ -121,9 +175,11 @@ let rec pp_tactic ~map_unicode_to_tex ~term_pp ~lazy_term_pp =
   | AutoBatch (_,params) -> "autobatch " ^ 
       pp_auto_params ~term_pp params
   | Assumption _ -> "assumption"
-  | Cases (_, term, pattern, specs) -> Printf.sprintf "cases " ^ term_pp term ^
-      pp_tactic_pattern pattern ^
-      pp_intros_specs "names " specs 
+  | Cases (_, term, pattern, specs) -> 
+      Printf.sprintf "cases %s %s%s" 
+      (term_pp term)
+      (pp_tactic_pattern pattern)
+      (pp_intros_specs "names " specs)
   | Change (_, where, with_what) ->
       Printf.sprintf "change %s with %s" (pp_tactic_pattern where) (lazy_term_pp with_what)
   | Clear (_,ids) -> Printf.sprintf "clear %s" (pp_hyps ids)
@@ -249,26 +305,38 @@ let pp_arg ~term_pp arg =
    else
      "(" ^ s ^ ")"
   
+let pp_nmacro = function
+  | NCheck (_, term) -> Printf.sprintf "ncheck %s" (CicNotationPp.pp_term term)
+  | Screenshot (_, name) -> Printf.sprintf "screenshot \"%s\"" name
+;;
+
 let pp_macro ~term_pp ~lazy_term_pp = 
   let term_pp = pp_arg ~term_pp in
-  let style_pp = function
-     | Declarative         -> ""
-     | Procedural None     -> "procedural "
-     | Procedural (Some i) -> Printf.sprintf "procedural %u " i
-  in
-  let prefix_pp prefix = 
-     if prefix = "" then "" else Printf.sprintf " \"%s\"" prefix
-  in
   let flavour_pp = function
-     | None                   -> ""
-     | Some `Definition       -> " as definition"
-     | Some `MutualDefinition -> " as mutual"
-     | Some `Fact             -> " as fact"
-     | Some `Lemma            -> " as lemma"
-     | Some `Remark           -> " as remark"
-     | Some `Theorem          -> " as theorem"
-     | Some `Variant          -> " as variant"
-     | Some `Axiom            -> " as axiom"
+     | `Definition       -> "definition"
+     | `Fact             -> "fact"
+     | `Lemma            -> "lemma"
+     | `Remark           -> "remark"
+     | `Theorem          -> "theorem"
+     | `Variant          -> "variant"
+     | `Axiom            -> "axiom"
+     | `MutualDefinition -> assert false
+  in
+  let pp_inline_params l =
+     let pp_param = function
+        | IPPrefix prefix -> "prefix = \"" ^ prefix ^ "\""
+	| IPAs flavour  -> flavour_pp flavour
+	| IPCoercions   -> "coercions"
+	| IPDebug debug -> "debug = " ^ string_of_int debug
+	| IPProcedural  -> "procedural"
+	| IPNoDefaults  -> "nodefaults"
+	| IPDepth depth -> "depth = " ^ string_of_int depth
+	| IPLevel level -> "level = " ^ string_of_int level
+	| IPComments    -> "comments"
+	| IPCR          -> "cr"
+     in
+     let s = String.concat " " (List.map pp_param l) in
+     if s = "" then s else " " ^ s
   in
   let pp_reduction_kind = pp_reduction_kind ~term_pp:lazy_term_pp in
   function 
@@ -285,8 +353,8 @@ let pp_macro ~term_pp ~lazy_term_pp =
   | Hint (_, true) -> "hint rewrite"
   | Hint (_, false) -> "hint"
   | AutoInteractive (_,params) -> "auto " ^ pp_auto_params ~term_pp params
-  | Inline (_, style, suri, prefix, flavour) ->  
-      Printf.sprintf "inline %s\"%s\"%s%s" (style_pp style) suri (prefix_pp prefix) (flavour_pp flavour) 
+  | Inline (_, suri, params) ->  
+      Printf.sprintf "inline \"%s\"%s" suri (pp_inline_params params) 
 
 let pp_associativity = function
   | Gramext.LeftA -> "left associative"
@@ -295,11 +363,6 @@ let pp_associativity = function
 
 let pp_precedence i = Printf.sprintf "with precedence %d" i
 
-let pp_dir_opt = function
-  | None -> ""
-  | Some `LeftToRight -> "> "
-  | Some `RightToLeft -> "< "
-
 let pp_default what uris = 
   Printf.sprintf "default \"%s\" %s" what
     (String.concat " " (List.map UriManager.string_of_uri uris))
@@ -308,18 +371,38 @@ let pp_coercion ~term_pp t do_composites arity saturations=
    Printf.sprintf "coercion %s %d %d %s"
     (term_pp t) arity saturations
     (if do_composites then "" else "nocomposites")
+
+let pp_ncommand = function
+  | UnificationHint (_,t, n) -> 
+      "unification hint " ^ string_of_int n ^ " " ^ CicNotationPp.pp_term t
+  | NDiscriminator (_,_)
+  | NInverter (_,_,_,_,_)
+  | NObj (_,_)
+  | NUnivConstraint (_) -> "not supported"
+  | NCoercion (_) -> "not supported"
+  | NQed (_) -> "nqed"
+  | NCopy (_,name,uri,map) -> 
+      "copy " ^ name ^ " from " ^ NUri.string_of_uri uri ^ " with " ^ 
+        String.concat " and " 
+          (List.map 
+            (fun (a,b) -> NUri.string_of_uri a ^ " ↦ " ^ NUri.string_of_uri b) 
+            map)
+;;
     
 let pp_command ~term_pp ~obj_pp = function
   | Index (_,_,uri) -> "Indexing " ^ UriManager.string_of_uri uri
+  | Select (_,uri) -> "Selecting " ^ UriManager.string_of_uri uri
   | Coercion (_, t, do_composites, i, j) ->
      pp_coercion ~term_pp t do_composites i j
   | PreferCoercion (_,t) -> 
      "prefer coercion " ^ term_pp t
-  | UnificationHint (_,t) -> 
-      "unification hint " ^ term_pp t
+  | Inverter (_,n,ty,params) ->
+     "inverter " ^ n ^ " for " ^ term_pp ty ^ " " ^ List.fold_left (fun acc x -> acc ^ (match x with true -> "%" | _ -> "?")) "" params
   | Default (_,what,uris) -> pp_default what uris
   | Drop _ -> "drop"
-  | Include (_,path) -> "include \"" ^ path ^ "\""
+  | Include (_,true,`OldAndNew,path) -> "include \"" ^ path ^ "\""
+  | Include (_,false,`OldAndNew,path) -> "include source \"" ^ path ^ "\""
+  | Include (_,_,`New,path) -> "RECURSIVELY INCLUDING " ^ path
   | Obj (_,obj) -> obj_pp obj
   | Qed _ -> "qed"
   | Relation (_,id,a,aeq,refl,sym,trans) ->
@@ -335,8 +418,9 @@ let pp_command ~term_pp ~obj_pp = function
        | None -> "")
   | Print (_,s) -> "print " ^ s
   | Set (_, name, value) -> Printf.sprintf "set \"%s\" \"%s\"" name value
+  | Pump (_) -> "not supported"
 
-let pp_punctuation_tactical ~term_pp ~lazy_term_pp =
+let pp_punctuation_tactical =
   function
   | Dot _ -> "."
   | Semicolon _ -> ";"
@@ -346,7 +430,7 @@ let pp_punctuation_tactical ~term_pp ~lazy_term_pp =
   | Wildcard _ -> "*:"
   | Merge _ -> "]"
 
-let pp_non_punctuation_tactical ~term_pp ~lazy_term_pp =
+let pp_non_punctuation_tactical =
   function
   | Focus (_, goals) ->
       Printf.sprintf "focus %s" (String.concat " " (List.map string_of_int goals))
@@ -355,16 +439,20 @@ let pp_non_punctuation_tactical ~term_pp ~lazy_term_pp =
 
 let pp_executable ~map_unicode_to_tex ~term_pp ~lazy_term_pp ~obj_pp =
   function
+  | NMacro (_, macro) -> pp_nmacro macro ^ "."
   | Macro (_, macro) -> pp_macro ~term_pp ~lazy_term_pp macro ^ "."
   | Tactic (_, Some tac, punct) ->
-      pp_tactic ~map_unicode_to_tex ~lazy_term_pp ~term_pp tac
-      ^ pp_punctuation_tactical ~lazy_term_pp ~term_pp punct
+      pp_tactic ~map_unicode_to_tex ~term_pp ~lazy_term_pp tac
+      ^ pp_punctuation_tactical punct
   | Tactic (_, None, punct) ->
-     pp_punctuation_tactical ~lazy_term_pp ~term_pp punct
+     pp_punctuation_tactical punct
+  | NTactic (_,tacl) ->
+      String.concat " " (List.map (pp_ntactic ~map_unicode_to_tex) tacl)
   | NonPunctuationTactical (_, tac, punct) ->
-     pp_non_punctuation_tactical ~lazy_term_pp ~term_pp tac
-     ^ pp_punctuation_tactical ~lazy_term_pp ~term_pp punct
+     pp_non_punctuation_tactical tac
+     ^ pp_punctuation_tactical punct
   | Command (_, cmd) -> pp_command ~term_pp ~obj_pp cmd ^ "."
+  | NCommand (_, cmd) -> pp_ncommand cmd ^ "."
                       
 let pp_comment ~map_unicode_to_tex ~term_pp ~lazy_term_pp ~obj_pp =
   function
