@@ -78,10 +78,5 @@ val pp_statement:
   map_unicode_to_tex:bool ->
     string
 
-val pp_punctuation_tactical:
-  term_pp:('term -> string) ->
-  lazy_term_pp:('lazy_term -> string) ->
-  ('term, 'lazy_term, 'term GrafiteAst.reduction, string)
-  GrafiteAst.punctuation_tactical ->
-    string
+val pp_punctuation_tactical: GrafiteAst.punctuation_tactical -> string
 
