@@ -23,27 +23,15 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: cicNotation2.ml 6144 2006-02-22 22:39:47Z zacchiro $ *)
+(* $Id: cicNotation2.ml 9896 2009-06-18 09:07:37Z sacerdot $ *)
 
-let load_notation ~include_paths fname =
+let load_notation status ~include_paths fname =
   let ic = open_in fname in
   let lexbuf = Ulexing.from_utf8_channel ic in
-  let status = ref LexiconEngine.initial_status in
+  let status = ref status in
   try
    while true do
     status := fst (GrafiteParser.parse_statement ~include_paths lexbuf !status)
    done;
    assert false
   with End_of_file -> close_in ic; !status
-
-let parse_environment ~include_paths str =
- let lexbuf = Ulexing.from_utf8_string str in
- let status = ref LexiconEngine.initial_status in
- try
-  while true do
-   status := fst (GrafiteParser.parse_statement ~include_paths lexbuf !status)
-  done;
-  assert false
- with End_of_file ->
-  !status.LexiconEngine.aliases,
-  !status.LexiconEngine.multi_aliases
