@@ -23,15 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(** Note: notation is also loaded, but it cannot be undone since the
-    notation_ids part of the status is thrown away;
-    so far this function is useful only in Whelp *)
-val parse_environment:
- include_paths:string list ->
- string ->
-  LexiconAst.alias_spec DisambiguateTypes.Environment.t *
-  LexiconAst.alias_spec list DisambiguateTypes.Environment.t
-
-
 (** @param fname file from which load notation *)
-val load_notation: include_paths:string list -> string -> LexiconEngine.status
+val load_notation:
+ #LexiconEngine.status as 'status -> include_paths:string list -> string ->
+  'status
