@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: grafiteDisambiguate.ml 9486 2009-01-13 10:54:34Z tassi $ *)
+(* $Id: grafiteDisambiguate.ml 10533 2009-10-28 15:26:49Z sacerdot $ *)
 
 exception BaseUriNotSetYet
 
@@ -78,6 +78,7 @@ let ncic_mk_choice = function
            (NCic.Appl l)::tl -> NCic.Appl (l@tl) | l -> NCic.Appl l)
         ~term_of_uri:(fun uri ->
            fst (OCic2NCic.convert_term uri (CicUtil.term_of_uri uri)))
+        ~term_of_nref:(fun nref -> NCic.Const nref)
        name dsc
   | LexiconAst.Number_alias (_, dsc) -> 
        let desc,f = DisambiguateChoices.lookup_num_by_dsc dsc in
@@ -89,8 +90,13 @@ let ncic_mk_choice = function
   | LexiconAst.Ident_alias (name, uri) -> 
      uri, `Sym_interp 
       (fun l->assert(l = []);
-      let uri = UriManager.uri_of_string uri in
-       fst (OCic2NCic.convert_term uri (CicUtil.term_of_uri uri)))
+        try
+         let nref = NReference.reference_of_string uri in
+          NCic.Const nref
+        with
+         NReference.IllFormedReference _ ->
+          let uri = UriManager.uri_of_string uri in
+           fst (OCic2NCic.convert_term uri (CicUtil.term_of_uri uri)))
 ;;
 
 
@@ -152,18 +158,53 @@ let lookup_in_library
   | DisambiguateTypes.Num instance -> mk_num_alias instance
 ;;
 
+let nlookup_in_library 
+  interactive_user_uri_choice input_or_locate_uri item 
+=
+  match item with
+  | DisambiguateTypes.Id id -> 
+     (try
+       let references = NCicLibrary.resolve id in
+        List.map
+         (fun u -> LexiconAst.Ident_alias (id,NReference.string_of_reference u)
+         ) references @
+        lookup_in_library interactive_user_uri_choice input_or_locate_uri item
+      with
+       NCicEnvironment.ObjectNotFound _ ->
+        lookup_in_library interactive_user_uri_choice input_or_locate_uri item)
+  | _ -> lookup_in_library interactive_user_uri_choice input_or_locate_uri item 
+;;
+
+let fix_instance item l =
+ match item with
+    DisambiguateTypes.Symbol (_,n) ->
+     List.map
+      (function
+          LexiconAst.Symbol_alias (s,_,d) -> LexiconAst.Symbol_alias (s,n,d)
+        | _ -> assert false
+      ) l
+  | DisambiguateTypes.Num n ->
+     List.map
+      (function
+          LexiconAst.Number_alias (_,d) -> LexiconAst.Number_alias (n,d)
+        | _ -> assert false
+      ) l
+  | DisambiguateTypes.Id _ -> l
+;;
+
+
   (** @param term not meaningful when context is given *)
-let disambiguate_term goal text prefix_len lexicon_status_ref context metasenv
+let disambiguate_term expty text prefix_len lexicon_status_ref context metasenv
 term =
   let lexicon_status = !lexicon_status_ref in
   let (diff, metasenv, subst, cic, _) =
     singleton "first"
       (CicDisambiguate.disambiguate_term
-        ~aliases:lexicon_status.LexiconEngine.aliases
-        ?goal ~universe:(Some lexicon_status.LexiconEngine.multi_aliases)
+        ~aliases:lexicon_status#lstatus.LexiconEngine.aliases
+        ~expty ~universe:(Some lexicon_status#lstatus.LexiconEngine.multi_aliases)
         ~lookup_in_library
         ~mk_choice:cic_mk_choice
-        ~mk_implicit
+        ~mk_implicit ~fix_instance
         ~description_of_alias:LexiconAst.description_of_alias
         ~context ~metasenv ~subst:[] (text,prefix_len,term))
   in
@@ -172,12 +213,32 @@ term =
   metasenv,(*subst,*) cic
 ;;
 
+let disambiguate_nterm expty estatus context metasenv subst thing
+=
+  let diff, metasenv, subst, cic =
+    singleton "first"
+      (NCicDisambiguate.disambiguate_term
+        ~rdb:estatus
+        ~aliases:estatus#lstatus.LexiconEngine.aliases
+        ~expty 
+        ~universe:(Some estatus#lstatus.LexiconEngine.multi_aliases)
+        ~lookup_in_library:nlookup_in_library
+        ~mk_choice:ncic_mk_choice
+        ~mk_implicit ~fix_instance
+        ~description_of_alias:LexiconAst.description_of_alias
+        ~context ~metasenv ~subst thing)
+  in
+  let estatus = LexiconEngine.set_proof_aliases estatus diff in
+   metasenv, subst, estatus, cic
+;;
+
+
   (** disambiguate_lazy_term (circa): term -> (unit -> status) * lazy_term
    * rationale: lazy_term will be invoked in different context to obtain a term,
    * each invocation will disambiguate the term and can add aliases. Once all
    * disambiguations have been performed, the first returned function can be
    * used to obtain the resulting aliases *)
-let disambiguate_lazy_term goal text prefix_len lexicon_status_ref term =
+let disambiguate_lazy_term expty text prefix_len lexicon_status_ref term =
   (fun context metasenv ugraph ->
     let lexicon_status = !lexicon_status_ref in
     let (diff, metasenv, _, cic, ugraph) =
@@ -185,12 +246,12 @@ let disambiguate_lazy_term goal text prefix_len lexicon_status_ref term =
         (CicDisambiguate.disambiguate_term 
           ~lookup_in_library
           ~mk_choice:cic_mk_choice
-          ~mk_implicit
+          ~mk_implicit ~fix_instance
           ~description_of_alias:LexiconAst.description_of_alias
-          ~initial_ugraph:ugraph ~aliases:lexicon_status.LexiconEngine.aliases
-          ~universe:(Some lexicon_status.LexiconEngine.multi_aliases)
-          ~context ~metasenv ~subst:[] ?goal
-          (text,prefix_len,term)) in
+          ~initial_ugraph:ugraph ~aliases:lexicon_status#lstatus.LexiconEngine.aliases
+          ~universe:(Some lexicon_status#lstatus.LexiconEngine.multi_aliases)
+          ~context ~metasenv ~subst:[] 
+          (text,prefix_len,term) ~expty) in
     let lexicon_status = LexiconEngine.set_proof_aliases lexicon_status diff in
     lexicon_status_ref := lexicon_status;
     cic, metasenv, ugraph)
@@ -214,6 +275,20 @@ let disambiguate_pattern
   (wanted, hyp_paths, goal_path)
 ;;
 
+type pattern = 
+  CicNotationPt.term Disambiguate.disambiguator_input option * 
+  (string * NCic.term) list * NCic.term option
+
+let disambiguate_npattern (text, prefix_len, (wanted, hyp_paths, goal_path)) =
+  let interp path = NCicDisambiguate.disambiguate_path path in
+  let goal_path = HExtlib.map_option interp goal_path in
+  let hyp_paths = List.map (fun (name, path) -> name, interp path) hyp_paths in
+  let wanted = 
+    match wanted with None -> None | Some x -> Some (text,prefix_len,x)
+  in
+   (wanted, hyp_paths, goal_path)
+;;
+
 let disambiguate_reduction_kind text prefix_len lexicon_status_ref = function
   | `Unfold (Some t) ->
       let t = 
@@ -253,7 +328,10 @@ let rec disambiguate_tactic
   lexicon_status_ref context metasenv goal (text,prefix_len,tactic) 
 =
   let disambiguate_term_hint = 
-    disambiguate_term goal text prefix_len lexicon_status_ref in
+    let _,_,expty = 
+      List.find (fun (x,_,_) -> Some x = goal) metasenv
+    in
+    disambiguate_term (Some expty) text prefix_len lexicon_status_ref in
   let disambiguate_term = 
     disambiguate_term None text prefix_len lexicon_status_ref in
   let disambiguate_pattern = 
@@ -559,31 +637,33 @@ let rec disambiguate_tactic
           | `Proof as t -> metasenv,t in
         metasenv,GrafiteAst.RewritingStep (loc, cic, cic', cic'', cont)   
 
-let disambiguate_obj lexicon_status ?baseuri metasenv (text,prefix_len,obj) =
+let disambiguate_obj estatus ?baseuri metasenv (text,prefix_len,obj) =
   let uri =
-   match obj with
-    | CicNotationPt.Inductive (_,(name,_,_,_)::_)
-    | CicNotationPt.Record (_,name,_,_) ->
-       (match baseuri with
-         | Some baseuri ->
-            Some (UriManager.uri_of_string (baseuri ^ "/" ^ name ^ ".ind"))
-         | None -> raise BaseUriNotSetYet)
-    | CicNotationPt.Inductive _ -> assert false
-    | CicNotationPt.Theorem _ -> None in
- let try_new cic =
+   let baseuri = 
+     match baseuri with Some x -> x | None -> raise BaseUriNotSetYet
+   in
+   let name = 
+     match obj with
+     | CicNotationPt.Inductive (_,(name,_,_,_)::_)
+     | CicNotationPt.Record (_,name,_,_) -> name ^ ".ind"
+     | CicNotationPt.Theorem (_,name,_,_,_) -> name ^ ".con"
+     | CicNotationPt.Inductive _ -> assert false
+   in
+     UriManager.uri_of_string (baseuri ^ "/" ^ name)
+  in
+(*
+ let _try_new cic =
   (NCicLibrary.clear_cache ();
    NCicEnvironment.invalidate ();
    OCic2NCic.clear ();
-   (match obj with
-      CicNotationPt.Theorem (_,_,ty,_) ->
    let graph = 
      match cic with
-     | Cic.CurrentProof (_,metasenv, _, ty,_,_) ->
+     | Some (Cic.CurrentProof (_,metasenv, _, ty,_,_)) ->
          let _, ugraph = 
            CicTypeChecker.type_of_aux' metasenv [] ty CicUniv.empty_ugraph
          in
            ugraph
-     | Cic.Constant (_,_, ty,_,_) ->
+     | Some (Cic.Constant (_,_, ty,_,_)) ->
          let _, ugraph = 
                  CicTypeChecker.type_of_aux' [] [] ty CicUniv.empty_ugraph
          in
@@ -607,7 +687,7 @@ let disambiguate_obj lexicon_status ?baseuri metasenv (text,prefix_len,obj) =
                 let _,g = CicTypeChecker.typecheck uri in
                 CicUniv.merge_ugraphs ~base_ugraph:graph ~increment:(g,uri))
              graph l)
-       graph (CoercDb.to_list ())
+       graph (CoercDb.to_list (CoercDb.dump ()))
    in
    ignore(CicUniv.do_rank graph);
 
@@ -625,80 +705,114 @@ let disambiguate_obj lexicon_status ?baseuri metasenv (text,prefix_len,obj) =
    let time = Unix.gettimeofday () in
        (try
          (match 
-          NCicDisambiguate.disambiguate_term
-           ~lookup_in_library:lookup_in_library
+          NCicDisambiguate.disambiguate_obj
+           ~rdb:estatus
+           ~lookup_in_library:nlookup_in_library
            ~description_of_alias:LexiconAst.description_of_alias
            ~mk_choice:ncic_mk_choice
            ~mk_implicit
-           ~coercion_db:(NCicCoercion.db ())
-           ~context:[] ~metasenv:[] ~subst:[]
-           ~aliases:lexicon_status.LexiconEngine.aliases
-           ~universe:(Some lexicon_status.LexiconEngine.multi_aliases) 
-           (text,prefix_len,ty)
+           ~uri:(OCic2NCic.nuri_of_ouri uri)
+           ~aliases:estatus#lstatus.LexiconEngine.aliases
+           ~universe:(Some estatus#lstatus.LexiconEngine.multi_aliases)
+           (text,prefix_len,obj)
          with
-         | [_,metasenv,subst,ty],_ ->
+         | [_,_,_,obj],_ ->
           let time = Unix.gettimeofday () -. time in
+(*           NCicTypeChecker.typecheck_obj obj; *)
           prerr_endline ("NUOVA DISAMBIGUAZIONE OK: "^ string_of_float time);
-          prerr_endline (NCicPp.ppterm ~metasenv ~subst ~context:[] ty)
+(*
+          let obj = 
+            let u,i,m,_,o = obj in
+            u,i,m,[],o
+          in
+*)
+          prerr_endline (NCicPp.ppobj obj)
          | _ ->
           prerr_endline ("NUOVA DISAMBIGUAZIONE AMBIGUO!!!!!!!!!  "))
        with 
        | MultiPassDisambiguator.DisambiguationError (_,s) ->
-        prerr_endline ("ERRORE NUOVA DISAMBIGUAZIONE:\n" ^
+        prerr_endline ("ERRORE NUOVA DISAMBIGUAZIONE ("
+          ^UriManager.string_of_uri uri^
+          "):\n" ^
          String.concat "\n" 
           (List.map (fun _,_,x,_ -> snd (Lazy.force x)) (List.flatten s)))
 (*        | exn -> prerr_endline (Printexc.to_string exn) *)
        )
-    | _ -> ())
   )
  in 
-
-
-(*
- try
-  let time = Unix.gettimeofday () in 
 *)
 
 
+ try
+(*   let time = Unix.gettimeofday () in  *)
+
+
   let (diff, metasenv, _, cic, _) =
     singleton "third"
       (CicDisambiguate.disambiguate_obj 
         ~lookup_in_library 
         ~mk_choice:cic_mk_choice
-        ~mk_implicit
+        ~mk_implicit ~fix_instance
         ~description_of_alias:LexiconAst.description_of_alias
-        ~aliases:lexicon_status.LexiconEngine.aliases
-        ~universe:(Some lexicon_status.LexiconEngine.multi_aliases) ~uri 
-        (text,prefix_len,obj)) in
+        ~aliases:estatus#lstatus.LexiconEngine.aliases
+        ~universe:(Some estatus#lstatus.LexiconEngine.multi_aliases) 
+        ~uri:(Some uri)
+        (text,prefix_len,obj)) 
+  in
 
 
 (*
   let time = Unix.gettimeofday () -. time in
-  prerr_endline ("VECCHIA DISAMBIGUAZIONE: " ^ string_of_float time);
-  try_new cic;
+  prerr_endline ("VECCHIA DISAMBIGUAZIONE ("^
+    UriManager.string_of_uri uri ^"): " ^ string_of_float time);
 *)
+(*    try_new (Some cic);   *)
 
 
+  let estatus = LexiconEngine.set_proof_aliases estatus diff in
+   estatus, metasenv, cic
 
-  let lexicon_status = LexiconEngine.set_proof_aliases lexicon_status diff in
-  lexicon_status, metasenv, cic
-
-
-(*
  with 
  | Sys.Break as exn -> raise exn
  | exn ->
-   try_new (Cic.InductiveDefinition ([],[],0,[])); 
+(*    try_new None; *)
    raise exn
-*)
-
+;;
 
+let disambiguate_nobj estatus ?baseuri (text,prefix_len,obj) =
+  let uri =
+   let baseuri = 
+     match baseuri with Some x -> x | None -> raise BaseUriNotSetYet
+   in
+   let name = 
+     match obj with
+     | CicNotationPt.Inductive (_,(name,_,_,_)::_)
+     | CicNotationPt.Record (_,name,_,_) -> name ^ ".ind"
+     | CicNotationPt.Theorem (_,name,_,_,_) -> name ^ ".con"
+     | CicNotationPt.Inductive _ -> assert false
+   in
+     UriManager.uri_of_string (baseuri ^ "/" ^ name)
+  in
+  let diff, _, _, cic =
+   singleton "third"
+    (NCicDisambiguate.disambiguate_obj
+      ~lookup_in_library:nlookup_in_library
+      ~description_of_alias:LexiconAst.description_of_alias
+      ~mk_choice:ncic_mk_choice
+      ~mk_implicit ~fix_instance
+      ~uri:(OCic2NCic.nuri_of_ouri uri)
+      ~rdb:estatus
+      ~aliases:estatus#lstatus.LexiconEngine.aliases
+      ~universe:(Some estatus#lstatus.LexiconEngine.multi_aliases) 
+      (text,prefix_len,obj)) in
+  let estatus = LexiconEngine.set_proof_aliases estatus diff in
+   estatus, cic
 ;;
   
-let disambiguate_command lexicon_status ?baseuri metasenv (text,prefix_len,cmd)=
+let disambiguate_command estatus ?baseuri metasenv (text,prefix_len,cmd)=
   match cmd with
    | GrafiteAst.Index(loc,key,uri) ->
-       let lexicon_status_ref = ref lexicon_status in 
+       let lexicon_status_ref = ref estatus in 
        let disambiguate_term =
          disambiguate_term None text prefix_len lexicon_status_ref [] in
        let disambiguate_term_option metasenv =
@@ -709,38 +823,42 @@ let disambiguate_command lexicon_status ?baseuri metasenv (text,prefix_len,cmd)=
                  metasenv, Some t
        in
        let metasenv,key = disambiguate_term_option metasenv key in
-       !lexicon_status_ref, metasenv,GrafiteAst.Index(loc,key,uri)
+        !lexicon_status_ref,metasenv,GrafiteAst.Index(loc,key,uri)
+   | GrafiteAst.Select (loc,uri) -> 
+        estatus, metasenv, GrafiteAst.Select(loc,uri)
+   | GrafiteAst.Pump(loc,i) -> 
+        estatus, metasenv, GrafiteAst.Pump(loc,i)
    | GrafiteAst.PreferCoercion (loc,t) -> 
-       let lexicon_status_ref = ref lexicon_status in 
+       let lexicon_status_ref = ref estatus in 
        let disambiguate_term =
          disambiguate_term None text prefix_len lexicon_status_ref [] in
       let metasenv,t = disambiguate_term metasenv t in
-      !lexicon_status_ref, metasenv, GrafiteAst.PreferCoercion (loc,t)
+       !lexicon_status_ref, metasenv, GrafiteAst.PreferCoercion (loc,t)
    | GrafiteAst.Coercion (loc,t,b,a,s) -> 
-       let lexicon_status_ref = ref lexicon_status in 
+       let lexicon_status_ref = ref estatus in 
        let disambiguate_term =
          disambiguate_term None text prefix_len lexicon_status_ref [] in
       let metasenv,t = disambiguate_term metasenv t in
-      !lexicon_status_ref, metasenv, GrafiteAst.Coercion (loc,t,b,a,s)
-   | GrafiteAst.UnificationHint (loc, t) ->
-       let lexicon_status_ref = ref lexicon_status in 
-       let disambiguate_term =
+       !lexicon_status_ref, metasenv, GrafiteAst.Coercion (loc,t,b,a,s)
+   | GrafiteAst.Inverter (loc,n,indty,params) ->
+       let lexicon_status_ref = ref estatus in
+       let disambiguate_term = 
          disambiguate_term None text prefix_len lexicon_status_ref [] in
-      let metasenv,t = disambiguate_term metasenv t in
-      !lexicon_status_ref, metasenv, GrafiteAst.UnificationHint (loc,t)
+       let metasenv,indty = disambiguate_term metasenv indty in
+        !lexicon_status_ref, metasenv, GrafiteAst.Inverter (loc,n,indty,params)
    | GrafiteAst.Default _
    | GrafiteAst.Drop _
    | GrafiteAst.Include _
    | GrafiteAst.Print _
    | GrafiteAst.Qed _
    | GrafiteAst.Set _ as cmd ->
-       lexicon_status,metasenv,cmd
+       estatus,metasenv,cmd
    | GrafiteAst.Obj (loc,obj) ->
-       let lexicon_status,metasenv,obj =
-        disambiguate_obj lexicon_status ?baseuri metasenv (text,prefix_len,obj)in
-       lexicon_status, metasenv, GrafiteAst.Obj (loc,obj)
+       let estatus,metasenv,obj =
+        disambiguate_obj estatus ?baseuri metasenv (text,prefix_len,obj)in
+       estatus, metasenv, GrafiteAst.Obj (loc,obj)
    | GrafiteAst.Relation (loc,id,a,aeq,refl,sym,trans) ->
-      let lexicon_status_ref = ref lexicon_status in 
+      let lexicon_status_ref = ref estatus in 
       let disambiguate_term =
        disambiguate_term None text prefix_len lexicon_status_ref [] in
       let disambiguate_term_option metasenv =
