@@ -23,18 +23,13 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: test_dep.ml 7922 2007-11-25 13:12:25Z tassi $ *)
+(* $Id: test_dep.ml 8052 2008-01-10 22:32:03Z tassi $ *)
 
 let _ =
-  let ic = ref stdin in
+  let ic = ref "/dev/fd/0" in
   let usage = "test_coarse_parser [ file ]" in
-  let open_file fname =
-    if !ic <> stdin then close_in !ic;
-    ic := open_in fname
-  in
+  let open_file fname = ic := fname in
   Arg.parse [] open_file usage;
-  let deps =
-    DependenciesParser.parse_dependencies (Ulexing.from_utf8_channel !ic)
-  in
+  let deps = DependenciesParser.deps_of_file !ic in
   List.iter (fun dep -> print_endline (DependenciesParser.pp_dependency dep)) deps
 
