@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: test_parser.ml 7541 2007-07-19 12:07:44Z sacerdot $ *)
+(* $Id: test_parser.ml 9896 2009-06-18 09:07:37Z sacerdot $ *)
 
 open Printf
 
@@ -41,7 +41,8 @@ let dump_xml t id_to_uri fname =
   prerr_endline (sprintf "dumping MathML to %s ..." fname);
   flush stdout;
   let oc = open_out fname in
-  let markup = CicNotationPres.render id_to_uri t in
+  let markup =
+   CicNotationPres.render ~lookup_uri:(CicNotationPres.lookup_uri id_to_uri)t in
   let xml_stream = CicNotationPres.print_xml markup in
   Xml.pp_to_outchan xml_stream oc;
   close_out oc
@@ -66,7 +67,7 @@ let process_stream istream =
   let module G = GrafiteAst in
   let status =
    ref
-    (CicNotation2.load_notation
+    (CicNotation2.load_notation (new LexiconEngine.status)
       ~include_paths:[] (Helm_registry.get "notation.core_file"))
   in
     try
