@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: lexiconAst.ml 7718 2007-10-12 18:12:38Z sacerdot $ *)
+(* $Id: lexiconAst.ml 9287 2008-12-03 22:53:01Z sacerdot $ *)
 
 type direction = [ `LeftToRight | `RightToLeft ]
 
@@ -55,3 +55,8 @@ type command =
 (* composed magic: term + command magics. No need to change this value *)
 let magic = magic + 10000 * CicNotationPt.magic
 
+let description_of_alias =
+ function
+    Ident_alias (_,desc)
+  | Symbol_alias (_,_,desc)
+  | Number_alias (_,desc) -> desc
