@@ -25,20 +25,30 @@
 
 exception IncludedFileNotCompiled of string * string
 
-type status = {
+type lexicon_status = {
   aliases: LexiconAst.alias_spec DisambiguateTypes.Environment.t;
   multi_aliases: LexiconAst.alias_spec list DisambiguateTypes.Environment.t;
   lexicon_content_rev: LexiconMarshal.lexicon;
   notation_ids: CicNotation.notation_id list;      (** in-scope notation ids *)
 }
 
-val initial_status: status
+class type g_status =
+ object
+  method lstatus: lexicon_status
+ end
 
-val eval_command : status -> LexiconAst.command -> status
+class status :
+ object ('self)
+  inherit g_status
+  method set_lstatus: lexicon_status -> 'self
+  method set_lexicon_engine_status: #g_status -> 'self
+ end
+
+val eval_command : #status as 'status -> LexiconAst.command -> 'status
 
 val set_proof_aliases:
- status -> (DisambiguateTypes.domain_item * LexiconAst.alias_spec) list ->
-  status
+ #status as 'status ->
+  (DisambiguateTypes.domain_item * LexiconAst.alias_spec) list -> 'status
 
-(* this callback is called on every lexicon command *)
-val set_callback: (LexiconAst.command -> unit) -> unit 
+(* args: print function, message (may be empty), status *) 
+val dump_aliases: (string -> unit) -> string -> #status -> unit
