@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: lexiconMarshal.ml 5769 2006-01-08 18:00:22Z sacerdot $ *)
+(* $Id: lexiconMarshal.ml 10004 2009-07-07 01:49:29Z sacerdot $ *)
 
 type lexicon = LexiconAst.command list
 
@@ -45,6 +45,9 @@ let rehash_cmd_uris =
         function
         | CicNotationPt.UriPattern uri ->
             CicNotationPt.UriPattern (rehash_uri uri)
+        | CicNotationPt.NRefPattern (NReference.Ref (uri,spec)) ->
+           let uri = NCicLibrary.refresh_uri uri in
+            CicNotationPt.NRefPattern (NReference.reference_of_spec uri spec)
         | CicNotationPt.ApplPattern args ->
             CicNotationPt.ApplPattern (List.map aux args)
         | CicNotationPt.VarPattern _
