@@ -24,17 +24,18 @@
  *)
 
 val add_aliases_for_objs:
- LexiconEngine.status -> UriManager.uri list -> LexiconEngine.status
+ #LexiconEngine.status as 'status ->
+  [`Old of UriManager.uri list | `New of NUri.uri list]-> 'status
 
 val time_travel: 
-  present:LexiconEngine.status -> past:LexiconEngine.status -> unit
+  present:#LexiconEngine.status -> past:#LexiconEngine.status -> unit
 
   (** perform a diff between the aliases contained in two statuses, assuming
     * that the second one can only have more aliases than the first one
     * @return the list of aliases that should be added to aliases of from in
     * order to be equal to aliases of the second argument *)
 val alias_diff:
- from:LexiconEngine.status -> LexiconEngine.status ->
+ from:#LexiconEngine.status -> #LexiconEngine.status ->
   (DisambiguateTypes.domain_item * LexiconAst.alias_spec) list
 
 val push: unit -> unit
