@@ -23,13 +23,14 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: cicCoercion.ml 7616 2007-09-04 10:45:18Z sacerdot $ *)
+(* $Id: cicCoercion.ml 8795 2008-07-07 21:01:17Z tassi $ *)
 
 let close_coercion_graph_ref = ref
  (fun _ _ _ _ _ -> [] : 
    CoercDb.coerc_carr -> CoercDb.coerc_carr -> UriManager.uri -> int ->
    string (* baseuri *) ->
-     (CoercDb.coerc_carr * CoercDb.coerc_carr * UriManager.uri * int * Cic.obj * int) list)
+     (CoercDb.coerc_carr * CoercDb.coerc_carr * UriManager.uri * int * Cic.obj *
+     int * int) list)
 ;;
 
 let set_close_coercion_graph f = close_coercion_graph_ref := f;;
