@@ -71,8 +71,8 @@ module type Format =
     val build: options -> source_object -> bool
     val root_and_target_of: 
           options -> source_object -> 
-	   (* root, writeable target, read only target *)
-	   string option * target_object * target_object
+	   (* root, relative source, writeable target, read only target *)
+	   string option * source_object * target_object * target_object
     val mtime_of_source_object: source_object -> float option
     val mtime_of_target_object: target_object -> float option
     val is_readonly_buri_of: options -> source_object -> bool
@@ -99,6 +99,7 @@ val write_deps_file: string option -> (string * string list) list -> unit
 (* true if the argunent starts with a uri scheme prefix *)
 val is_uri: string -> bool
 
-val debug: bool ref
+(* Valid values: 0: no debug; 1: normal debug; > 1: extra debug *)
+val debug: int ref
 
 val time_stamp: string -> unit
