@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: libraryClean.ml 8052 2008-01-10 22:32:03Z tassi $ *)
+(* $Id: libraryClean.ml 9912 2009-06-19 12:42:05Z sacerdot $ *)
 
 open Printf
 
@@ -34,6 +34,8 @@ module HGT = Http_getter_types;;
 module HG = Http_getter;;
 module UM = UriManager;;
 
+let decompile = ref (fun ~baseuri -> assert false);;
+let set_decompile_cb f = decompile := f;;
 
 let strip_xpointer s = Pcre.replace ~pat:"#.*$" s ;;
 
@@ -230,6 +232,7 @@ let clean_baseuris ?(verbose=true) buris =
     List.iter (fun u -> debug_prerr (UriManager.string_of_uri u)) l; 
   List.iter
    (fun baseuri ->
+     !decompile ~baseuri;
      try 
       let obj_file =
        LibraryMisc.obj_file_of_baseuri ~must_exist:false ~writable:true ~baseuri
