@@ -9,17 +9,21 @@
      \ /   This software is distributed as is, NO WARRANTY.     
       V_______________________________________________________________ *)
 
-(* $Id: nCic.ml 9058 2008-10-13 17:42:30Z tassi $ *)
+(* $Id: nCic.ml 10511 2009-10-22 22:11:03Z tassi $ *)
 
 (********************************* TERMS ************************************)
 
-type universe = (bool * NUri.uri) list 
+type univ_algebra = [ `Type | `Succ | `CProp ]
+
+type universe = (univ_algebra * NUri.uri) list 
   (* Max of non-empty list of named universes, or their successor (when true) 
    * The empty list represents type0 *)
 
 type sort = Prop | Type of universe
 
-type implicit_annotation = [ `Closed | `Type | `Hole | `Term | `Typeof of int ]
+type implicit_annotation =
+ [ `Closed | `Type | `Hole | `Tagged of string | `Term | `Typeof of int | `Vector ]
+
 
 type lc_kind = Irl of int | Ctx of term list
 
@@ -52,11 +56,18 @@ type hypothesis = string * context_entry   (* name, entry *)
 
 type context = hypothesis list
 
-type conjecture = string option * context * term
+type meta_attr = 
+  [ `Name of string 
+  | `IsTerm | `IsType | `IsSort 
+  | `InScope | `OutScope of int]
+
+type meta_attrs = meta_attr list
+
+type conjecture =  meta_attrs * context * term
 
 type metasenv = (int * conjecture) list
 
-type subst_entry = string option * context * term * term (* name,ctx,bo,ty *)
+type subst_entry = meta_attrs * context * term * term (* name,ctx,bo,ty *)
 
 type substitution = (int * subst_entry) list
 
@@ -95,7 +106,7 @@ type ind_pragma = (* pragmatic of the object *)
 type generated = [ `Generated | `Provided ]
 
 type c_attr = generated * def_flavour * def_pragma
-type f_attr = generated * def_flavour
+type f_attr = generated * def_flavour * def_pragma
 type i_attr = generated * ind_pragma
 
  (* invariant: metasenv and substitution have disjoint domains *)
