@@ -13,22 +13,48 @@
 
 type db
 
+val set_convert_term:
+ (UriManager.uri -> Cic.term -> NCic.term * NCic.obj list) -> unit
+
+class type g_status =
+ object
+  inherit NCicUnifHint.g_status
+  method coerc_db: db
+ end
+
+class status :
+ object ('self)
+  inherit NCicUnifHint.status
+  inherit g_status
+  method coerc_db: db
+  method set_coerc_db: db -> 'self
+  method set_coercion_status: #g_status -> 'self
+ end
+
+val empty_db: db
+
 (* index (\x.c ?? x ??): A -> B
    index_coercion db c A B \arity_left(c ??x??) \position(x,??x??) 
 *)
 val index_coercion: 
-  db -> NCic.term -> NCic.term -> NCic.term -> int -> int -> db
+  #status as 'status -> string ->
+   NCic.term -> NCic.term -> NCic.term -> int -> int -> 'status
 
-  (* gets the old imperative coercion DB *)
-val db : unit -> db
-
-val empty_db : db
+  (* gets the old imperative coercion DB (list format) *)
+val index_old_db: CoercDb.coerc_db -> (#status as 'status) -> 'status
 
 val look_for_coercion:
-    db ->
+    #status ->
     NCic.metasenv -> NCic.substitution -> NCic.context -> 
     (* inferred type, expected type *)
     NCic.term -> NCic.term -> 
-      (* enriched metasenv, new term, its type, metavriable to
+      (* name, enriched metasenv, new term, its type, metavriable to
        * be unified with the old term *)
-      (NCic.metasenv * NCic.term * NCic.term * NCic.term) list
+      (string * NCic.metasenv * NCic.term * NCic.term * NCic.term) list
+
+(* returns (coercion,arity,arg) *)
+val match_coercion:
+ #status -> metasenv:NCic.metasenv -> subst:NCic.substitution ->
+  context:NCic.context -> NCic.term -> (NCic.term * int * int) option
+
+val generate_dot_file: #status -> Format.formatter -> unit
