@@ -13,17 +13,34 @@
 
 type db
 
+exception HintNotValid
+
+class type g_status =
+ object
+  method uhint_db: db
+ end
+
+class status :
+ object ('self)
+  inherit g_status
+  method set_uhint_db: db -> 'self
+  method set_unifhint_status: #g_status -> 'self
+ end
+
 val index_hint: 
-  db -> NCic.context -> NCic.term -> NCic.term -> db
+  #status as 'status -> NCic.context -> NCic.term -> NCic.term -> int -> 'status
 
-  (* gets the old imperative coercion DB *)
-val db : unit -> db
-val add_user_provided_hint : Cic.term -> unit 
-
-val empty_db : db
+val add_user_provided_hint :
+  #status as 'status -> NCic.term -> int -> 'status
 
 val look_for_hint:
-    db ->
+    #status ->
     NCic.metasenv -> NCic.substitution -> NCic.context -> 
     NCic.term -> NCic.term -> 
-      (NCic.metasenv * NCic.term * NCic.term) list
+      (NCic.metasenv * 
+        (NCic.term * NCic.term) * (NCic.term * NCic.term) list) list
+
+val eq_class_of:
+      #status -> NCic.term -> NCic.term list
+
+val generate_dot_file: #status -> Format.formatter -> unit
