@@ -153,3 +153,6 @@ let cache_size (_,oldcache) =
 let cache_clean (univ,oldcache) = 
   univ,List.filter (function (_,Succeded _) -> true | _ -> false) oldcache
 ;;
+let cache_reset_underinspection (u,c) =
+  u,List.filter (function (_,UnderInspection) -> false | _ -> true) c
+;;
