@@ -476,7 +476,7 @@ let number_if_already_defined buri name l =
  *)
 let close_coercion_graph src tgt uri saturations baseuri =
   (* check if the coercion already exists *)
-  let coercions = CoercDb.to_list () in
+  let coercions = CoercDb.to_list (CoercDb.dump ()) in
   let todo_list = get_closure_coercions src tgt (uri,saturations,0) coercions in
   debug_print (lazy("composed " ^ string_of_int (List.length todo_list)));
   let todo_list = filter_duplicates todo_list coercions in
