@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: continuationals.ml 7710 2007-10-09 11:01:34Z tassi $ *)
+(* $Id: continuationals.ml 9611 2009-03-25 20:41:04Z tassi $ *)
 
 open Printf
 
@@ -117,6 +117,10 @@ struct
   let of_metasenv metasenv =
     let goals = List.map (fun (g, _, _) -> g) metasenv in
     [ zero_pos goals, [], [], `NoTag ]
+  
+  let of_nmetasenv metasenv =
+    let goals = List.map (fun (g, _) -> g) metasenv in
+    [ zero_pos goals, [], [], `NoTag ]
 
   let head_switches =
     function
