@@ -30,11 +30,11 @@ val assume : string -> Cic.term -> ProofEngineTypes.tactic
 val suppose : Cic.term -> string -> Cic.term option -> ProofEngineTypes.tactic
 
 val by_just_we_proved :
- dbd:HSql.dbd -> universe:Universe.universe -> 
+ dbd:HSql.dbd -> automation_cache:AutomationCache.cache -> 
  just -> Cic.term -> string option -> Cic.term option ->
  ProofEngineTypes.tactic
 
-val bydone : dbd:HSql.dbd -> universe:Universe.universe ->
+val bydone : dbd:HSql.dbd -> automation_cache:AutomationCache.cache ->
   just -> ProofEngineTypes.tactic
 
 val we_need_to_prove :
@@ -51,15 +51,15 @@ val thesisbecomes : Cic.term -> ProofEngineTypes.tactic
 val case : string -> params:(string * Cic.term) list -> ProofEngineTypes.tactic
 
 val existselim :
-  dbd:HSql.dbd -> universe:Universe.universe -> just ->
+  dbd:HSql.dbd -> automation_cache:AutomationCache.cache -> just ->
   string -> Cic.term -> string -> Cic.lazy_term -> ProofEngineTypes.tactic
 
 val andelim :
- dbd:HSql.dbd -> universe:Universe.universe -> just ->
+ dbd:HSql.dbd -> automation_cache:AutomationCache.cache -> just ->
  string -> Cic.term -> string -> Cic.term -> ProofEngineTypes.tactic
 
 val rewritingstep :
- dbd:HSql.dbd -> universe:Universe.universe ->
+ dbd:HSql.dbd -> automation_cache:AutomationCache.cache ->
   (string option * Cic.term) option -> Cic.term ->
    [ `Term of Cic.term | `Auto of Auto.auto_params
    | `Proof  | `SolveWith of Cic.term] ->
