@@ -23,7 +23,7 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: fourierR.ml 8206 2008-03-11 22:02:58Z sacerdot $ *)
+(* $Id: fourierR.ml 9486 2009-01-13 10:54:34Z tassi $ *)
 
 
 (******************** THE FOURIER TACTIC ***********************)
@@ -439,8 +439,8 @@ let fourier_lineq lineq1 =
    let hvar=Hashtbl.create 50 in (* la table des variables des inquations *)
    List.iter (fun f ->
                Hashtbl.iter (fun x c ->
-                                 try (Hashtbl.find hvar x;())
-                                 with _-> nvar:=(!nvar)+1;
+                                 try ignore(Hashtbl.find hvar x)
+                                 with Not_found -> nvar:=(!nvar)+1;
                                              Hashtbl.add hvar x (!nvar);
                                           debug("aggiungo una var "^
                                            string_of_int !nvar^" per "^
