@@ -25,3 +25,6 @@
 
 (* $Id: primitiveTactics.ml 9014 2008-09-26 08:03:47Z tassi $ *)
 val init: unit -> unit
+val build_inverter: add_obj:(UriManager.uri -> Cic.obj -> 'b -> 'b * UriManager.uri list) ->
+                    'b -> UriManager.uri -> UriManager.uri -> bool list -> 
+                    'b * UriManager.uri list
