@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: metadataQuery.ml 9204 2008-11-16 10:08:58Z tassi $ *)
+(* $Id: metadataQuery.ml 9740 2009-05-08 14:50:27Z asperti $ *)
 
 open Printf
 
@@ -274,7 +274,9 @@ let signature_of metasenv goal =
   let ty_set = Constr.constants_of ty in
   let hyp_set = signature_of_hypothesis context metasenv in
   let set = Constr.UriManagerSet.union ty_set hyp_set in
-    close_with_constructors (close_with_types set metasenv context)
+    close_with_types
+     (close_with_constructors (close_with_types set metasenv context)
+	metasenv context)
     metasenv context
 
 
