@@ -1,4 +1,4 @@
-(* cOpyright (C) 2005, HELM Team.
+(* Copyright (C) 2005, HELM Team.
  * 
  * This file is part of HELM, an Hypertextual, Electronic
  * Library of Mathematics, developed at the Computer Science
@@ -47,30 +47,26 @@ and proof =
 and goal_proof = (rule * Utils.pos * int * Subst.substitution * Cic.term) list
 ;;
 (* the hashtbl eq_id -> proof, max_eq_id *)
-type equality_bag = (int,equality) Hashtbl.t * int ref
+module IntOt = struct type t = int let compare = Pervasives.compare end
+module M = Map.Make(IntOt)
+type equality_bag = equality M.t * int
 
 type goal = goal_proof * Cic.metasenv * Cic.term
 
 (* globals *)
-let mk_equality_bag () =
-  Hashtbl.create 1024, ref 0
-;;
+let mk_equality_bag () = M.empty, 10000 ;; 
 
-let freshid (_,i) =
-  incr i; !i
-;;
+let freshid (m,i) = (m,i+1), i+1 ;;
 
-let add_to_bag (id_to_eq,_) id eq =
-  Hashtbl.add id_to_eq id eq
-;;
+let add_to_bag (id_to_eq,i) id eq = M.add id eq id_to_eq,i ;;
 
 let uncomparable = fun _ -> 0
 
 let mk_equality bag (weight,p,(ty,l,r,o),m) =
-  let id = freshid bag in
+  let bag, id = freshid bag in
   let eq = (uncomparable,weight,p,(ty,l,r,o),m,id) in
-  add_to_bag bag id eq;
-  eq
+  let bag = add_to_bag bag id eq in
+  bag, eq
 ;;
 
 let mk_tmp_equality (weight,(ty,l,r,o),m) =
@@ -82,6 +78,11 @@ let mk_tmp_equality (weight,(ty,l,r,o),m) =
 let open_equality (_,weight,proof,(ty,l,r,o),m,id) = 
   (weight,proof,(ty,l,r,o),m,id)
 
+let id_of e = 
+  let _,_,_,_,id = open_equality e in id
+;;
+
+
 let string_of_rule = function
   | SuperpositionRight -> "SupR"
   | SuperpositionLeft -> "SupL"
@@ -117,8 +118,8 @@ let rec max_weight_in_proof ((id_to_eq,_) as bag) current =
   function
    | Exact _ -> current
    | Step (_, (_,id1,(_,id2),_)) ->
-       let eq1 = Hashtbl.find id_to_eq id1 in
-       let eq2 = Hashtbl.find id_to_eq id2 in  
+       let eq1 = M.find id1 id_to_eq in
+       let eq2 = M.find id2 id_to_eq in  
        let (w1,p1,(_,_,_,_),_,_) = open_equality eq1 in
        let (w2,p2,(_,_,_,_),_,_) = open_equality eq2 in
        let current = max current w1 in
@@ -129,7 +130,7 @@ let rec max_weight_in_proof ((id_to_eq,_) as bag) current =
 let max_weight_in_goal_proof ((id_to_eq,_) as bag) =
   List.fold_left 
     (fun current (_,_,id,_,_) ->
-       let eq = Hashtbl.find id_to_eq id in
+       let eq = M.find id id_to_eq in
        let (w,p,(_,_,_,_),_,_) = open_equality eq in
        let current = max current w in
        max_weight_in_proof bag current p)
@@ -140,10 +141,17 @@ let max_weight bag goal_proof proof =
 
 let proof_of_id (id_to_eq,_) id =
   try
-    let (_,p,(_,l,r,_),_,_) = open_equality (Hashtbl.find id_to_eq id) in
+    let (_,p,(_,l,r,_),_,_) = open_equality (M.find id id_to_eq) in
       p,l,r
   with
-      Not_found -> assert false
+      Not_found -> 
+              prerr_endline ("Unable to find the proof of " ^ string_of_int id);
+              assert false
+;;
+
+let is_in (id_to_eq,_) id = 
+  M.mem id id_to_eq
+;;
 
 
 let string_of_proof ?(names=[]) bag p gp = 
@@ -187,8 +195,8 @@ let rec depend ((id_to_eq,_) as bag) eq id seen =
       | Exact _ -> false,seen
       | Step (_,(_,id1,(_,id2),_)) ->
           let seen = ideq::seen in
-          let eq1 = Hashtbl.find id_to_eq id1 in
-          let eq2 = Hashtbl.find id_to_eq id2 in  
+          let eq1 = M.find id1 id_to_eq in
+          let eq2 = M.find id2 id_to_eq in  
           let b1,seen = depend bag eq1 id seen in
           if b1 then b1,seen else depend bag eq2 id seen
 ;;
@@ -627,7 +635,7 @@ let wfo bag goalproof proof id =
 let string_of_id (id_to_eq,_) names id = 
   if id = 0 then "" else 
   try
-    let (_,p,(t,l,r,_),m,_) = open_equality (Hashtbl.find id_to_eq id) in
+    let (_,p,(t,l,r,_),m,_) = open_equality (M.find id id_to_eq) in
     match p with
     | Exact t -> 
         Printf.sprintf "%d = %s: %s = %s [%s]" id
@@ -661,14 +669,6 @@ let pp_proof bag names goalproof proof subst id initial_goal =
   "\nand then subsumed by " ^ string_of_int id ^ " when " ^ Subst.ppsubst subst
 ;;
 
-module OT = 
-  struct
-    type t = int
-    let compare = Pervasives.compare
-  end
-
-module M = Map.Make(OT)
-
 let rec find_deps bag m i = 
   if M.mem i m then m
   else 
@@ -716,7 +716,6 @@ let topological_sort bag l =
   rc
 ;;
   
-
 (* returns the list of ids that should be factorized *)
 let get_duplicate_step_in_wfo bag l p =
   let ol = List.rev l in
@@ -896,11 +895,9 @@ let relocate newmeta menv to_be_relocated =
   let menv = Subst.apply_subst_metasenv subst (menv @ newmetasenv) in
   subst, menv, newmeta
 
-let fix_metas_goal newmeta goal =
+let fix_metas_goal (id_to_eq,newmeta) goal =
   let (proof, menv, ty) = goal in
-  let to_be_relocated = 
-    HExtlib.list_uniq (List.sort Pervasives.compare (Utils.metas_of_term ty))
-  in
+  let to_be_relocated = List.map (fun i ,_,_ -> i) menv in
   let subst, menv, newmeta = relocate newmeta menv to_be_relocated in
   let ty = Subst.apply_subst subst ty in
   let proof = 
@@ -908,20 +905,12 @@ let fix_metas_goal newmeta goal =
     | [] -> assert false (* is a nonsense to relocate the initial goal *)
     | (r,pos,id,s,p) :: tl -> (r,pos,id,Subst.concat subst s,p) :: tl
   in
-  newmeta+1,(proof, menv, ty)
+  (id_to_eq,newmeta+1),(proof, menv, ty)
 ;;
 
-let fix_metas bag newmeta eq = 
+let fix_metas (id_to_eq, newmeta) eq = 
   let w, p, (ty, left, right, o), menv,_ = open_equality eq in
-  let to_be_relocated = 
-  List.map (fun i ,_,_ -> i) menv 
-(*
-    HExtlib.list_uniq 
-      (List.sort Pervasives.compare 
-         (Utils.metas_of_term left @ Utils.metas_of_term right @
-         Utils.metas_of_term ty)) 
-*)
-  in
+  let to_be_relocated = List.map (fun i ,_,_ -> i) menv in
   let subst, metasenv, newmeta = relocate newmeta menv to_be_relocated in
   let ty = Subst.apply_subst subst ty in
   let left = Subst.apply_subst subst left in
@@ -932,8 +921,10 @@ let fix_metas bag newmeta eq =
         Step (Subst.concat s subst,(r,id1,(pos,id2), pred))
   in
   let p = fix_proof p in
-  let eq' = mk_equality bag (w, p, (ty, left, right, o), metasenv) in
-  newmeta+1, eq'  
+  let bag = id_to_eq, newmeta in
+  let bag, e = mk_equality bag (w, p, (ty, left, right, o), metasenv) in
+  bag, e
+;;
 
 exception NotMetaConvertible;;
 
@@ -1090,15 +1081,15 @@ let term_is_equality term =
   | _ -> false
 ;;
 
-let equality_of_term bag proof term =
+let equality_of_term bag proof term newmetas =
   match term with
   | Cic.Appl [Cic.MutInd (uri, _, _); ty; t1; t2] 
     when LibraryObjects.is_eq_URI uri ->
       let o = !Utils.compare_terms t1 t2 in
       let stat = (ty,t1,t2,o) in
       let w = Utils.compute_equality_weight stat in
-      let e = mk_equality bag (w, Exact proof, stat,[]) in
-      e
+      let bag, e = mk_equality bag (w, Exact proof, stat,newmetas) in
+      bag, e
   | _ ->
       raise TermIsNotAnEquality
 ;;
@@ -1152,12 +1143,12 @@ let symmetric bag eq_ty l id uri m =
     Exact (Cic.Appl
       [Cic.MutConstruct(uri,0,1,[]);eq_ty;l]) 
   in
-  let id1 = 
-    let eq = mk_equality bag (0,prefl,(eq_ty,l,l,Utils.Eq),m) in
+  let bag, id1 = 
+    let bag, eq = mk_equality bag (0,prefl,(eq_ty,l,l,Utils.Eq),m) in
     let (_,_,_,_,id) = open_equality eq in
-    id
+    bag, id
   in
-  Step(Subst.empty_subst,
+  bag, Step(Subst.empty_subst,
     (Demodulation,id1,(Utils.Left,id),pred))
 ;;
 
@@ -1170,8 +1161,7 @@ module IntSet = Set.Make(IntOT);;
 
 let n_purged = ref 0;;
 
-let collect ((id_to_eq,_) as bag) alive1 alive2 alive3 =
-(*   let _ = <:start<collect>> in *)
+let collect ((id_to_eq,maxmeta) as bag) alive1 alive2 alive3 =
   let deps_of id = 
     let p,_,_ = proof_of_id bag id in  
     match p with
@@ -1187,18 +1177,13 @@ let collect ((id_to_eq,_) as bag) alive1 alive2 alive3 =
   let alive_set = l_to_s (l_to_s (l_to_s IntSet.empty alive2) alive1) alive3 in
   let closed_alive_set = close alive_set in
   let to_purge = 
-    Hashtbl.fold 
+    M.fold 
       (fun k _ s -> 
         if not (IntSet.mem k closed_alive_set) then
           k::s else s) id_to_eq []
   in
   n_purged := !n_purged + List.length to_purge;
-  List.iter (Hashtbl.remove id_to_eq) to_purge;
-(*   let _ = <:stop<collect>> in ()   *)
-;;
-
-let id_of e = 
-  let _,_,_,_,id = open_equality e in id
+  List.fold_right M.remove to_purge id_to_eq, maxmeta
 ;;
 
 let get_stats () = "" 
@@ -1319,7 +1304,7 @@ let remove_names_in_context (set,subst) names =
 let string_of_id2 (id_to_eq,_) names nameset id = 
   if id = 0 then "" else 
   try
-    let (_,_,(_,l,r,_),_,_) = open_equality (Hashtbl.find id_to_eq id) in
+    let (_,_,(_,l,r,_),_,_) = open_equality (M.find id id_to_eq) in
     let nameset, l = freshname nameset l in
     let nameset, r = freshname nameset r in
     Printf.sprintf "%s = %s" (CicPp.pp l names) (CicPp.pp r names)
@@ -1379,3 +1364,16 @@ let draw_proof bag names goal_proof proof id =
   ignore(Unix.system "gv /tmp/matita_paramod.eps");
 ;;
 
+let saturate_term (id_to_eq, maxmeta) metasenv subst context term = 
+  let maxmeta = max maxmeta (CicMkImplicit.new_meta metasenv subst) in
+  let head, metasenv, args, newmeta =
+    TermUtil.saturate_term maxmeta metasenv context term 0
+  in
+  (id_to_eq, newmeta), head, metasenv, args
+;;
+
+let push_maxmeta (id_to_eq, maxmeta) m = id_to_eq, max maxmeta m ;;
+let filter_metasenv_gt_maxmeta (_,maxmeta) =
+  List.filter (fun (j,_,_) -> j >= maxmeta)
+;;
+let maxmeta = snd;;
