@@ -23,7 +23,7 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: utils.ml 9146 2008-11-04 15:13:34Z asperti $ *)
+(* $Id: utils.ml 9580 2009-03-10 15:52:26Z asperti $ *)
 
 let time = false;;
 let debug = false;;
@@ -114,6 +114,8 @@ let rec remove_local_context =
     | Cic.Meta (i,_) -> Cic.Meta (i,[])
     | Cic.Appl l ->
        Cic.Appl(List.map remove_local_context l)
+    | Cic.Prod (n,s,t) -> 
+       Cic.Prod (n,remove_local_context s, remove_local_context t)
     | t -> t 
 
 
