@@ -23,7 +23,7 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: primitiveTactics.ml 9536 2009-02-03 20:39:00Z tassi $ *)
+(* $Id: primitiveTactics.ml 9769 2009-05-13 18:44:42Z fguidi $ *)
 
 exception TheTypeOfTheCurrentGoalIsAMetaICannotChooseTheRightElimiantionPrinciple
 exception NotAnInductiveTypeToEliminate
@@ -249,17 +249,17 @@ let new_metasenv_and_unify_and_t newmeta' metasenv' subst context term' ty termt
   in
   subst,newmetasenv',t
 
-let rec count_prods context ty =
- match CicReduction.whd context ty with
-    Cic.Prod (n,s,t) -> 1 + count_prods (Some (n,Cic.Decl s)::context) t
+let rec count_prods subst context ty =
+ match CicReduction.whd ~subst context ty with
+    Cic.Prod (n,s,t) -> 1 + count_prods subst (Some (n,Cic.Decl s)::context) t
   | _ -> 0
 
-let apply_with_subst ~term ~subst ~maxmeta (proof, goal) =
+let apply_with_subst ~term ~maxmeta (proof, goal) =
   (* Assumption: The term "term" must be closed in the current context *)
  let module T = CicTypeChecker in
  let module R = CicReduction in
  let module C = Cic in
-  let (_,metasenv,_subst,_,_, _) = proof in
+  let (_,metasenv,subst,_,_, _) = proof in
   let metano,context,ty = CicUtil.lookup_meta goal metasenv in
   let newmeta = max (CicMkImplicit.new_meta metasenv subst) maxmeta in
    let exp_named_subst_diff,newmeta',newmetasenvfragment,term' =
@@ -296,11 +296,12 @@ let apply_with_subst ~term ~subst ~maxmeta (proof, goal) =
    in
    let metasenv' = metasenv@newmetasenvfragment in
    let termty,_ = 
-     CicTypeChecker.type_of_aux' metasenv' context term' CicUniv.oblivion_ugraph
+     CicTypeChecker.type_of_aux' 
+       metasenv' ~subst context term' CicUniv.oblivion_ugraph
    in
    let termty =
      CicSubstitution.subst_vars exp_named_subst_diff termty in
-   let goal_arity = count_prods context ty in
+   let goal_arity = count_prods subst context ty in
    let subst,newmetasenv',t = 
     let rec add_one_argument n =
      try
@@ -330,6 +331,10 @@ let apply_with_subst ~term ~subst ~maxmeta (proof, goal) =
      newmetasenv''
    in
    let subst = ((metano,(context,bo',ty))::subst) in
+   let newproof = 
+     let u,m,_,p,t,l = newproof in
+     u,m,subst,p,t,l
+   in
    subst,
    (newproof, List.map (function (i,_,_) -> i) new_uninstantiatedmetas),
    max maxmeta (CicMkImplicit.new_meta newmetasenv''' subst)
@@ -338,9 +343,12 @@ let apply_with_subst ~term ~subst ~maxmeta (proof, goal) =
 (* ALB *)
 let apply_with_subst ~term ?(subst=[]) ?(maxmeta=0) status =
   try
-(*     apply_tac_verbose ~term status *)
-    apply_with_subst ~term ~subst ~maxmeta status
-      (* TODO cacciare anche altre eccezioni? *)
+    let status = 
+      if subst <> [] then
+        let (u,m,_,p,t,l), g = status in (u,m,subst,p,t,l), g
+      else status
+    in
+     apply_with_subst ~term ~maxmeta status
   with 
   | CicUnification.UnificationFailure msg
   | CicTypeChecker.TypeCheckerFailure msg -> raise (PET.Fail msg)
@@ -462,26 +470,8 @@ let letin_tac ?(mk_fresh_name_callback=FreshNamesGenerator.mk_fresh_name ~subst:
  in
   PET.mk_tactic (letin_tac ~mk_fresh_name_callback term)
 
-  (** functional part of the "exact" tactic *)
-let exact_tac ~term =
- let exact_tac ~term (proof, goal) =
-  (* Assumption: the term bo must be closed in the current context *)
-  let (_,metasenv,_subst,_,_, _) = proof in
-  let metano,context,ty = CicUtil.lookup_meta goal metasenv in
-  let module T = CicTypeChecker in
-  let module R = CicReduction in
-  let ty_term,u = T.type_of_aux' metasenv context term CicUniv.oblivion_ugraph in
-  let b,_ = R.are_convertible context ty_term ty u in (* TASSI: FIXME *)
-  if b then
-   begin
-    let (newproof, metasenv') =
-      ProofEngineHelpers.subst_meta_in_proof proof metano term [] in
-    (newproof, [])
-   end
-  else
-   raise (PET.Fail (lazy "The type of the provided term is not the one expected."))
- in
-  PET.mk_tactic (exact_tac ~term)
+(* FG: exact_tac := apply_tac as in NTactics *)
+let exact_tac ~term = apply_tac ~term
 
 (* not really "primitive" tactics .... *)
   
@@ -500,7 +490,8 @@ let rec args_init n f =
    if n <= 0 then [] else f n :: args_init (pred n) f
 
 let mk_predicate_for_elim 
- ~context ~metasenv ~ugraph ~goal ~arg ~using ~cpattern ~args_no = 
+ ~context ~metasenv ~subst ~ugraph ~goal ~arg ~using ~cpattern ~args_no 
+= 
    let instantiated_eliminator =
       let f n = if n = 1 then arg else C.Implicit None in
       C.Appl (using :: args_init args_no f)
@@ -514,39 +505,39 @@ let mk_predicate_for_elim
       | _                              -> assert false
    in
 (* let _, upto = PEH.split_with_whd (List.nth splits pred_pos) in *)
-   let rec mk_pred metasenv context' pred arg' cpattern' = function
-      | []           -> metasenv, pred, arg'
+   let rec mk_pred metasenv subst context' pred arg' cpattern' = function
+      | []           -> metasenv, subst, pred, arg'
       | arg :: tail -> 
 (* FG: we find the predicate for the eliminator as in the rewrite tactic ****)
-	 let argty, _ugraph = TC.type_of_aux' metasenv context arg ugraph in
-         let argty = CicReduction.whd context argty in         
+	 let argty, _ = TC.type_of_aux' metasenv ~subst context arg ugraph in
+         let argty = CicReduction.whd ~subst context argty in         
          let fresh_name = 
             FreshNamesGenerator.mk_fresh_name 
-            ~subst:[] metasenv context' C.Anonymous ~typ:argty in
+            ~subst metasenv context' C.Anonymous ~typ:argty in
 	 let hyp = Some (fresh_name, C.Decl argty) in
          let lazy_term c m u =  
           let distance  = List.length c - List.length context in
            S.lift distance arg, m, u in
          let pattern = Some lazy_term, [], Some cpattern' in
          let subst, metasenv, _ugraph, _conjecture, selected_terms =
-          ProofEngineHelpers.select ~metasenv ~ugraph
+          ProofEngineHelpers.select ~subst ~metasenv ~ugraph
            ~conjecture:(0, context, pred) ~pattern in
          let metasenv = MS.apply_subst_metasenv subst metasenv in  
          let map (_context_of_t, t) l = t :: l in
          let what = List.fold_right map selected_terms [] in
          let arg' = MS.apply_subst subst arg' in
-         let argty = MS.apply_subst subst argty in
          let pred = PER.replace_with_rel_1_from ~equality:(==) ~what 1 pred in
          let pred = MS.apply_subst subst pred in
-         let pred = C.Lambda (fresh_name, argty, pred) in
+	 let pred = C.Lambda (fresh_name, C.Implicit None, pred) in
 	 let cpattern' = C.Lambda (C.Anonymous, C.Implicit None, cpattern') in
-         mk_pred metasenv (hyp :: context') pred arg' cpattern' tail 
+         mk_pred metasenv subst (hyp :: context') pred arg' cpattern' tail 
    in
-   let metasenv, pred, arg = 
-      mk_pred metasenv context goal arg cpattern (List.rev actual_args)
+   let metasenv, subst, pred, arg = 
+      mk_pred metasenv subst context goal arg cpattern (List.rev actual_args)
    in
+   HLog.debug ("PREDICATE CONTEXT:\n" ^ CicPp.ppcontext ~metasenv context);
    HLog.debug ("PREDICATE: " ^ CicPp.ppterm ~metasenv pred ^ " ARGS: " ^ String.concat " " (List.map (CicPp.ppterm ~metasenv) actual_args));
-   metasenv, pred, arg, actual_args
+   metasenv, subst, pred, arg, actual_args
 
 let beta_after_elim_tac upto predicate =
    let beta_after_elim_tac status =
@@ -630,10 +621,10 @@ let generalize_tac
    let (proof, goal) = status in
    let module C = Cic in
    let module T = Tacticals in
-    let uri,metasenv,_subst,pbo,pty, attrs = proof in
+    let uri,metasenv,subst,pbo,pty, attrs = proof in
     let (_,context,ty) as conjecture = CicUtil.lookup_meta goal metasenv in
     let subst,metasenv,u,selected_hyps,terms_with_context =
-     ProofEngineHelpers.select ~metasenv ~ugraph:CicUniv.oblivion_ugraph
+     ProofEngineHelpers.select ~metasenv ~subst ~ugraph:CicUniv.oblivion_ugraph
       ~conjecture ~pattern in
     let context = CicMetaSubst.apply_subst_context subst context in
     let metasenv = CicMetaSubst.apply_subst_metasenv subst metasenv in
@@ -660,7 +651,7 @@ let generalize_tac
            context_of_t, t context_of_t metasenv u
        | (context_of_t, t)::_, None -> context_of_t, (t, metasenv, u)
      in
-      let t,subst,metasenv' =
+      let t,e_subst,metasenv' =
        try
         CicMetaSubst.delift_rels [] metasenv
          (List.length context_of_t - List.length context) t
@@ -670,7 +661,7 @@ let generalize_tac
       in
        (*CSC: I am not sure about the following two assertions;
          maybe I need to propagate the new subst and metasenv *)
-       assert (subst = []);
+       assert (e_subst = []);
        assert (metasenv' = metasenv);
        let typ,u = CicTypeChecker.type_of_aux' ~subst metasenv context t u in
         u,typ,t,metasenv
@@ -704,7 +695,7 @@ let generalize_tac
          else
           u1
       ) u terms_with_context) ;
-    let status = (uri,metasenv',_subst,pbo,pty, attrs),goal in
+    let status = (uri,metasenv',subst,pbo,pty, attrs),goal in
     let proof,goals =
      PET.apply_tactic 
       (T.thens 
@@ -724,7 +715,7 @@ let generalize_tac
             T.id_tac])
         status
     in
-     let _,metasenv'',_subst,_,_, _ = proof in
+     let _,metasenv'',_,_,_, _ = proof in
       (* CSC: the following is just a bad approximation since a meta
          can be closed and then re-opened! *)
       (proof,
@@ -780,7 +771,7 @@ let pattern_after_generalize_pattern_tac (tp, hpatterns, cpattern) =
 let elim_tac ?using ?(pattern = PET.conclusion_pattern None) term = 
  let elim_tac pattern (proof, goal) =
    let ugraph = CicUniv.oblivion_ugraph in
-   let curi, metasenv, _subst, proofbo, proofty, attrs = proof in
+   let curi, metasenv, subst, proofbo, proofty, attrs = proof in
    let conjecture = CicUtil.lookup_meta goal metasenv in
    let metano, context, ty = conjecture in 
    let pattern = pattern_after_generalize_pattern_tac pattern in
@@ -788,8 +779,8 @@ let elim_tac ?using ?(pattern = PET.conclusion_pattern None) term =
     match pattern with 
       | None, [], Some cpattern -> cpattern
       | _ -> raise (PET.Fail (lazy "not implemented")) in    
-    let termty,_ugraph = TC.type_of_aux' metasenv context term ugraph in
-    let termty = CicReduction.whd context termty in
+    let termty,_ugraph = TC.type_of_aux' metasenv ~subst context term ugraph in
+    let termty = CicReduction.whd ~subst context termty in
     let termty, metasenv', arguments, _fresh_meta =
      TermUtil.saturate_term
       (ProofEngineHelpers.new_meta_of_proof proof) metasenv context termty 0 in
@@ -811,7 +802,7 @@ let elim_tac ?using ?(pattern = PET.conclusion_pattern None) term =
             name
         | _ -> assert false
       in
-      let ty_ty,_ugraph = TC.type_of_aux' metasenv' context ty ugraph in
+      let ty_ty,_ugraph = TC.type_of_aux' metasenv' ~subst context ty ugraph in
       let ext =
        match ty_ty with
           C.Sort C.Prop -> "_ind"
@@ -828,7 +819,7 @@ let elim_tac ?using ?(pattern = PET.conclusion_pattern None) term =
          | Some t -> t 
        in
        let ety, _ugraph = 
-         TC.type_of_aux' metasenv' context eliminator_ref ugraph in
+         TC.type_of_aux' metasenv' ~subst context eliminator_ref ugraph in
 (* FG: ADDED PART ***********************************************************)
 (* FG: we can not assume eliminator is the default eliminator ***************)
    let splits, args_no = PEH.split_with_whd (context, ety) in
@@ -837,13 +828,13 @@ let elim_tac ?using ?(pattern = PET.conclusion_pattern None) term =
       | _, C.Appl (C.Rel i :: _) when i > 1 && i <= args_no -> i
       | _ -> raise NotAnEliminator
    in
-   let metasenv', pred, term, actual_args = match pattern with 
+   let metasenv', subst, pred, term, actual_args = match pattern with 
       | None, [], Some (C.Implicit (Some `Hole)) ->
-         metasenv', C.Implicit None, term, []
+         metasenv', subst, C.Implicit None, term, []
       | _                                        ->
          mk_predicate_for_elim 
 	    ~args_no ~context ~ugraph ~cpattern
-	    ~metasenv:metasenv' ~arg:term ~using:eliminator_ref ~goal:ty
+	    ~metasenv:metasenv' ~subst ~arg:term ~using:eliminator_ref ~goal:ty
    in
 (* FG: END OF ADDED PART ****************************************************)
       let term_to_refine =
@@ -853,28 +844,33 @@ let elim_tac ?using ?(pattern = PET.conclusion_pattern None) term =
          in
          C.Appl (eliminator_ref :: args_init args_no f)
       in
-      let refined_term,_refined_termty,metasenv'',_ugraph = 
-         CicRefine.type_of_aux' metasenv' context term_to_refine ugraph
+      let refined_term,_refined_termty,metasenv'',subst,_ugraph = 
+         CicRefine.type_of metasenv' subst context term_to_refine ugraph
+      in
+      let ipred = match refined_term with
+         | C.Appl ts -> List.nth ts (List.length ts - pred_pos)
+	 | _         -> assert false
       in
       let new_goals =
          ProofEngineHelpers.compare_metasenvs
             ~oldmetasenv:metasenv ~newmetasenv:metasenv''
       in
-      let proof' = curi,metasenv'',_subst,proofbo,proofty, attrs in
+      let proof' = curi,metasenv'',subst,proofbo,proofty, attrs in
       let proof'', new_goals' =
          PET.apply_tactic (apply_tac ~term:refined_term) (proof',goal)
       in
       (* The apply_tactic can have closed some of the new_goals *)
       let patched_new_goals =
-         let (_,metasenv''',_subst,_,_, _) = proof'' in
+         let (_,metasenv''',_,_,_, _) = proof'' in
          List.filter
             (function i -> List.exists (function (j,_,_) -> j=i) metasenv''')
 	    new_goals @ new_goals'
       in
       let res = proof'', patched_new_goals in
       let upto = List.length actual_args in
-      if upto = 0 then res else 
-      let continuation = beta_after_elim_tac upto pred in
+      if upto = 0 then res else
+(* FG: we use ipred (instantiated pred) instead of pred (not instantiated) *)
+      let continuation = beta_after_elim_tac upto ipred in
       let dummy_status = proof,goal in
       PET.apply_tactic
          (T.then_ ~start:(PET.mk_tactic (fun _ -> res)) ~continuation)
