@@ -23,12 +23,13 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: proofEngineTypes.ml 7777 2007-10-29 19:11:26Z fguidi $ *)
+(* $Id: proofEngineTypes.ml 9014 2008-09-26 08:03:47Z tassi $ *)
 
   (**
     current proof (proof uri * metas * (in)complete proof * term to be prooved)
   *)
-type proof = UriManager.uri option * Cic.metasenv * Cic.substitution * Cic.term * Cic.term * Cic.attribute list
+type proof = 
+  UriManager.uri option * Cic.metasenv * Cic.substitution * Cic.term Lazy.t * Cic.term * Cic.attribute list
   (** current goal, integer index *)
 type goal = int
 type status = proof * goal
@@ -45,7 +46,8 @@ let initial_status ty metasenv attrs =
   let newmeta_idx = aux 0 metasenv in
   let _subst = [] in
   let proof =
-    None, (newmeta_idx, [], ty) :: metasenv, _subst, Cic.Meta (newmeta_idx, []), ty, attrs
+    None, (newmeta_idx, [], ty) :: metasenv, _subst, 
+    lazy (Cic.Meta (newmeta_idx, [])), ty, attrs
   in
   (proof, newmeta_idx)
 
