@@ -23,7 +23,7 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: variousTactics.ml 8684 2008-06-08 13:44:20Z sacerdot $ *)
+(* $Id: variousTactics.ml 9707 2009-04-28 13:59:26Z tassi $ *)
 
 
 (* TODO se ce n'e' piu' di una, prende la prima che trova... sarebbe meglio
@@ -38,7 +38,7 @@ let assumption_tac =
   let module R = CicReduction in
   let module S = CicSubstitution in
   let module PT = PrimitiveTactics in
-  let _,metasenv,_subst,_,_, _ = proof in
+  let _,metasenv,_,_,_, _ = proof in
   let _,context,ty = CicUtil.lookup_meta goal metasenv in
   let rec find n = function 
       hd::tl -> 
