@@ -1,6 +1,8 @@
 (* OPTIONS *)
 let tptppath = ref "./";;
+let ng = ref false;;
 let spec = [
+  ("-ng",Arg.Set ng,"Matita ng syntax");
   ("-tptppath", 
       Arg.String (fun x -> tptppath := x), 
       "Where to find the Axioms/ and Problems/ directory")
@@ -17,6 +19,6 @@ let _ =
       exit 1
     end;
   print_endline 
-    (Tptp2grafite.tptp2grafite ~filename:!inputfile ~tptppath:!tptppath ());
+    (Tptp2grafite.tptp2grafite ~filename:!inputfile ~tptppath:!tptppath ~ng:!ng ());
   exit 0
 
