@@ -23,7 +23,7 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: uriManager.ml 6638 2006-07-19 10:43:43Z zacchiro $ *)
+(* $Id: uriManager.ml 9819 2009-06-03 08:56:58Z sacerdot $ *)
 
 (*
  * "cic:/a/b/c.con" => ("cic:/a/b/c.con", id )
@@ -191,6 +191,25 @@ let bodyuri_of_uri (uri, _) =
     None
 ;;
 
+let ind_uri_split ((s, _) as uri) =
+    let noxp = strip_xpointer uri in
+    try 
+      (let arg_index = String.rindex s '(' in
+       try 
+         (let ty_index = String.index_from s arg_index '/' in
+          try 
+            (let k_index = String.index_from s (ty_index+1) '/' in
+             let tyno = int_of_string (String.sub s (ty_index + 1) (k_index - ty_index - 1)) in
+             let kno = int_of_string (String.sub s (k_index + 1) (String.length s - k_index - 2)) in
+             noxp, Some tyno, Some kno)
+          with Not_found -> 
+            let tyno = int_of_string (String.sub s (ty_index + 1) (String.length s - ty_index - 2)) in
+            noxp, Some tyno, None)
+       with Not_found -> noxp, None, None
+      )
+    with Not_found -> noxp, None, None
+;;
+
 (* these are bugged!
  * we should remove _types, _univ, _ann all toghether *)
 let innertypesuri_of_uri (uri, _) =
