@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: dump_moo.ml 6836 2006-09-27 09:49:48Z sacerdot $ *)
+(* $Id: dump_moo.ml 9486 2009-01-13 10:54:34Z tassi $ *)
 
 open Printf
 
@@ -52,8 +52,8 @@ let _ =
           (fun cmd ->
             printf "  %s\n%!"
               (GrafiteAstPp.pp_command
-                ~term_pp:(fun _ -> assert false)
-                ~obj_pp:(fun _ -> assert false) cmd))
+                ~term_pp:(fun t -> CicPp.ppterm t)
+                ~obj_pp:(fun _ -> "OBJ") cmd))
           commands;
       end)
     (List.rev !moos)
