@@ -27,6 +27,7 @@
 
 type status =
     Cic.context *
-    (int * Cic.term * bool * int * (int * Cic.term) list) list *
-    (int * Cic.term * int) list * Cic.term list
+    (int * Cic.term * bool * int * (int * Cic.term Lazy.t) list) list *
+    (int * Cic.term * int) list * Cic.term Lazy.t list
 val auto_dialog : (unit -> status) -> unit
+val set_font_size: int -> unit
