@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: matitaGui.ml 9500 2009-01-16 13:28:08Z tassi $ *)
+(* $Id: matitaGui.ml 10387M 2009-12-01 22:51:56Z (local) $ *)
 
 open Printf
 
@@ -67,13 +67,13 @@ class console ~(buffer: GText.buffer) () =
   end
         
 let clean_current_baseuri grafite_status = 
-  LibraryClean.clean_baseuris [GrafiteTypes.get_baseuri grafite_status]
+  LibraryClean.clean_baseuris [grafite_status#baseuri]
 
-let save_moo lexicon_status grafite_status = 
+let save_moo grafite_status = 
   let script = MatitaScript.current () in
-  let baseuri = GrafiteTypes.get_baseuri grafite_status in
+  let baseuri = grafite_status#baseuri in
   let no_pstatus = 
-    grafite_status.GrafiteTypes.proof_status = GrafiteTypes.No_proof 
+    grafite_status#proof_status = GrafiteTypes.No_proof 
   in
   match script#bos, script#eos, no_pstatus with
   | true, _, _ -> ()
@@ -85,10 +85,11 @@ let save_moo lexicon_status grafite_status =
        LibraryMisc.lexicon_file_of_baseuri 
          ~must_exist:false ~baseuri ~writable:true
      in
-     GrafiteMarshal.save_moo moo_fname
-       grafite_status.GrafiteTypes.moo_content_rev;
+     GrafiteMarshal.save_moo moo_fname grafite_status#moo_content_rev;
      LexiconMarshal.save_lexicon lexicon_fname
-       lexicon_status.LexiconEngine.lexicon_content_rev
+      grafite_status#lstatus.LexiconEngine.lexicon_content_rev;
+     NCicLibrary.Serializer.serialize ~baseuri:(NUri.uri_of_string baseuri)
+      grafite_status#dump
   | _ -> clean_current_baseuri grafite_status 
 ;;
     
@@ -204,7 +205,7 @@ class interpErrorModel =
 exception UseLibrary;;
 
 let rec interactive_error_interp ~all_passes
-  (source_buffer:GSourceView.source_buffer) notify_exn offset errorll filename
+  (source_buffer:GSourceView2.source_buffer) notify_exn offset errorll filename
 = 
   (* hook to save a script for each disambiguation error *)
   if false then
@@ -373,12 +374,12 @@ class gui () =
     [ main#mainWinEventBox ]
   in
   let console = new console ~buffer:main#logTextView#buffer () in
-  let (source_view: GSourceView.source_view) =
-    GSourceView.source_view
+  let (source_view: GSourceView2.source_view) =
+    GSourceView2.source_view
       ~auto_indent:true
-      ~insert_spaces_instead_of_tabs:true ~tabs_width:2
-      ~margin:80 ~show_margin:true
-      ~smart_home_end:true
+      ~insert_spaces_instead_of_tabs:true ~tab_width:2
+      ~right_margin_position:80 ~show_right_margin:true
+      ~smart_home_end:`AFTER
       ~packing:main#scriptScrolledWin#add
       ()
   in
@@ -574,11 +575,15 @@ class gui () =
           source_view#misc#grab_focus ()
       in
       connect_menu_item main#undoMenuItem safe_undo;
+(*CSC: XXX
       ignore(source_view#source_buffer#connect#can_undo
         ~callback:main#undoMenuItem#misc#set_sensitive);
+*) main#undoMenuItem#misc#set_sensitive true;
       connect_menu_item main#redoMenuItem safe_redo;
+(*CSC: XXX
       ignore(source_view#source_buffer#connect#can_redo
         ~callback:main#redoMenuItem#misc#set_sensitive);
+*) main#redoMenuItem#misc#set_sensitive true;
       ignore(source_view#connect#after#populate_popup
        ~callback:(fun pre_menu ->
          let menu = new GMenu.menu pre_menu in
@@ -802,33 +807,33 @@ class gui () =
           else main#tacticsButtonsHandlebox#misc#hide ())
         ~check:main#menuitemPalette;
       connect_button main#butImpl_intro
-        (fun () -> source_buffer#insert "apply rule (⇒_i […] (…));\n");
+        (fun () -> source_buffer#insert "apply rule (⇒#i […] (…));\n");
       connect_button main#butAnd_intro
         (fun () -> source_buffer#insert 
-          "apply rule (∧_i (…) (…));\n\t[\n\t|\n\t]\n");
+          "apply rule (∧#i (…) (…));\n\t[\n\t|\n\t]\n");
       connect_button main#butOr_intro_left
-        (fun () -> source_buffer#insert "apply rule (∨_i_l (…));\n");
+        (fun () -> source_buffer#insert "apply rule (∨#i_l (…));\n");
       connect_button main#butOr_intro_right
-        (fun () -> source_buffer#insert "apply rule (∨_i_r (…));\n");
+        (fun () -> source_buffer#insert "apply rule (∨#i_r (…));\n");
       connect_button main#butNot_intro
-        (fun () -> source_buffer#insert "apply rule (¬_i […] (…));\n");
+        (fun () -> source_buffer#insert "apply rule (¬#i […] (…));\n");
       connect_button main#butTop_intro
-        (fun () -> source_buffer#insert "apply rule (⊤_i);\n");
+        (fun () -> source_buffer#insert "apply rule (⊤#i);\n");
       connect_button main#butImpl_elim
         (fun () -> source_buffer#insert 
-          "apply rule (⇒_e (…) (…));\n\t[\n\t|\n\t]\n");
+          "apply rule (⇒#e (…) (…));\n\t[\n\t|\n\t]\n");
       connect_button main#butAnd_elim_left
-        (fun () -> source_buffer#insert "apply rule (∧_e_l (…));\n");
+        (fun () -> source_buffer#insert "apply rule (∧#e_l (…));\n");
       connect_button main#butAnd_elim_right
-        (fun () -> source_buffer#insert "apply rule (∧_e_r (…));\n");
+        (fun () -> source_buffer#insert "apply rule (∧#e_r (…));\n");
       connect_button main#butOr_elim
         (fun () -> source_buffer#insert 
-          "apply rule (∨_e (…) […] (…) […] (…));\n\t[\n\t|\n\t|\n\t]\n");
+          "apply rule (∨#e (…) […] (…) […] (…));\n\t[\n\t|\n\t|\n\t]\n");
       connect_button main#butNot_elim
         (fun () -> source_buffer#insert 
-          "apply rule (¬_e (…) (…));\n\t[\n\t|\n\t]\n");
+          "apply rule (¬#e (…) (…));\n\t[\n\t|\n\t]\n");
       connect_button main#butBot_elim
-        (fun () -> source_buffer#insert "apply rule (⊥_e (…));\n");
+        (fun () -> source_buffer#insert "apply rule (⊥#e (…));\n");
       connect_button main#butRAA
         (fun () -> source_buffer#insert "apply rule (RAA […] (…));\n");
       connect_button main#butUseLemma
@@ -837,14 +842,14 @@ class gui () =
         (fun () -> source_buffer#insert "apply rule (discharge […]);\n");
       
       connect_button main#butForall_intro
-        (fun () -> source_buffer#insert "apply rule (∀_i {…} (…));\n");
+        (fun () -> source_buffer#insert "apply rule (∀#i {…} (…));\n");
       connect_button main#butForall_elim
-        (fun () -> source_buffer#insert "apply rule (∀_e {…} (…));\n");
+        (fun () -> source_buffer#insert "apply rule (∀#e {…} (…));\n");
       connect_button main#butExists_intro
-        (fun () -> source_buffer#insert "apply rule (∃_i {…} (…));\n");
+        (fun () -> source_buffer#insert "apply rule (∃#i {…} (…));\n");
       connect_button main#butExists_elim
         (fun () -> source_buffer#insert 
-          "apply rule (∃_e (…) {…} […] (…));\n\t[\n\t|\n\t]\n");
+          "apply rule (∃#e (…) {…} […] (…));\n\t[\n\t|\n\t]\n");
 
     
       (* TO BE REMOVED *)
@@ -882,13 +887,16 @@ class gui () =
           else raise exn);
         (* script *)
       let _ =
-        match GSourceView.source_language_from_file BuildTimeConf.lang_file with
+        let source_language_manager =
+         GSourceView2.source_language_manager ~default:true in
+        source_language_manager#set_search_path[BuildTimeConf.runtime_base_dir];
+        match source_language_manager#language "grafite" with
         | None ->
-            HLog.warn (sprintf "can't load language file %s"
-              BuildTimeConf.lang_file)
-        | Some matita_lang ->
+            HLog.warn(sprintf "can't load a language file for \"grafite\" in %s"
+              BuildTimeConf.runtime_base_dir)
+        | Some x as matita_lang ->
             source_buffer#set_language matita_lang;
-            source_buffer#set_highlight true
+            source_buffer#set_highlight_syntax true
       in
       let disableSave () =
         (s())#assignFileName None;
@@ -913,14 +921,13 @@ class gui () =
         else saveAsScript ()
       in
       let abandon_script () =
-        let lexicon_status = (s ())#lexicon_status in
         let grafite_status = (s ())#grafite_status in
         if source_view#buffer#modified then
           (match ask_unsaved main#toplevel with
           | `YES -> saveScript ()
           | `NO -> ()
           | `CANCEL -> raise MatitaTypes.Cancel);
-        save_moo lexicon_status grafite_status
+        save_moo grafite_status
       in
       let loadScript () =
         let script = s () in 
@@ -969,12 +976,12 @@ class gui () =
           match ask_unsaved main#toplevel with
           | `YES -> 
                saveScript ();
-               save_moo script#lexicon_status script#grafite_status;
+               save_moo script#grafite_status;
                GMain.Main.quit ()
           | `NO -> GMain.Main.quit ()
           | `CANCEL -> ()
         else 
-          (save_moo script#lexicon_status script#grafite_status;
+          (save_moo script#grafite_status;
           GMain.Main.quit ()));
       connect_button main#scriptAdvanceButton advance;
       connect_button main#scriptRetractButton retract;
@@ -995,6 +1002,10 @@ class gui () =
         (fun _ -> 
           let c = MatitaMathView.cicBrowser () in
           c#load (`About `Coercions));
+      connect_menu_item main#showHintsDbMenuItem 
+        (fun _ -> 
+          let c = MatitaMathView.cicBrowser () in
+          c#load (`About `Hints));
       connect_menu_item main#showAutoGuiMenuItem 
         (fun _ -> MatitaAutoGui.auto_dialog Auto.get_auto_status);
       connect_menu_item main#showTermGrammarMenuItem 
@@ -1285,8 +1296,8 @@ class gui () =
       self#main#saveMenuItem#misc#set_sensitive true
         
     method console = console
-    method sourceView: GSourceView.source_view =
-      (source_view: GSourceView.source_view)
+    method sourceView: GSourceView2.source_view =
+      (source_view: GSourceView2.source_view)
     method fileSel = fileSel
     method findRepl = findRepl
     method main = main
