@@ -43,10 +43,10 @@ val interactive_uri_choice:
   ?hide_uri_entry:bool -> ?hide_try:bool -> ?ok_label:string ->
   ?ok_action:[`AUTO|`SELECT] ->
   ?copy_cb:(string -> unit) -> unit ->
-    GrafiteDisambiguator.choose_uris_callback
+  id:'a -> UriManager.uri list -> UriManager.uri list
 
   (** @raise MatitaTypes.Cancel *)
 val interactive_interp_choice:
   unit ->
-    GrafiteDisambiguator.choose_interp_callback
+    DisambiguateTypes.interactive_interpretation_choice_type
 
