@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: matitaMisc.ml 8052 2008-01-10 22:32:03Z tassi $ *)
+(* $Id: matitaMisc.ml 9685 2009-04-21 19:49:59Z fguidi $ *)
 
 open Printf
 
@@ -155,9 +155,32 @@ let list_tl_at ?(equality=(==)) e l =
   aux l
 
 let shutup () = 
-  HLog.set_log_callback (fun _ _ -> ());
+  HLog.set_log_callback (fun _ _ -> ())
 (*
   let out = open_out "/dev/null" in
   Unix.dup2 (Unix.descr_of_out_channel out) (Unix.descr_of_out_channel stderr)
 *)
               
+(* FG: out_preamble *********************************************************)
+
+let out_comment och s =
+   let s = if s <> "" && s.[0] = '*' then "#" ^ s else s in
+   Printf.fprintf och "%s%s%s\n\n" "(*" s "*)" 
+ 
+let out_line_comment och s =
+   let l = 70 - String.length s in
+   let s = Printf.sprintf " %s %s" s (String.make l '*') in
+   out_comment och s
+
+let out_preamble och =
+   let rt_base_dir = Filename.dirname Sys.argv.(0) in
+   let path = Filename.concat rt_base_dir "matita.ma.templ" in
+   let lines = 14 in
+   let ich = open_in path in
+   let rec print i =
+      if i > 0 then 
+         let s = input_line ich in
+         begin Printf.fprintf och "%s\n" s; print (pred i) end
+   in 
+   print lines;
+   out_line_comment och "This file was automatically generated: do not edit"
