@@ -23,10 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* this callback is called on the expansion of every inline macro *)
-val set_callback: (string -> unit) -> unit 
-
 module Make : sig
-        val make: string -> string list -> bool
+   val make: string -> string list -> bool
 end
 
