@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: matitaclean.ml 8164 2008-02-22 22:25:27Z fguidi $ *)
+(* $Id: matitaclean.ml 9918 2009-06-19 15:49:45Z sacerdot $ *)
 
 open Printf
 
@@ -80,7 +80,8 @@ let clean_all () =
       ignore 
        (Sys.command ("find " ^ xmldir ^ 
         " -type d -exec rmdir -p {} \\; 2> /dev/null"))) 
-    prefixes
+    prefixes;
+  ignore (Sys.command ("rm -rf " ^ Helm_registry.get "matita.basedir"))
 ;;
 
 let main () =
